# -*- coding: utf-8 -*-
import wx
from wx.stc import *
from app.handlers import EditorHandlerBase
import app.resources as rc

MARGIN_LINE_NUMBERS = 0
MARK_MARGIN = 1
MARGIN_FOLD = 2

STC_MASK_MARKERS = ~STC_MASK_FOLDERS

MARK_BOOKMARK = 0
MARK_BREAKPOINT = 1
DEBUG_CURRENT = 32

#missing indicators
STC_INDIC_FULLBOX = 16

#undefined wrappers
SCI_ANNOTATIONSETTEXT = 2540
SCI_ANNOTATIONGETTEXT = 2541
SCI_ANNOTATIONSETSTYLE = 2542
SCI_ANNOTATIONGETSTYLE = 2543
SCI_ANNOTATIONSETSTYLES = 2544
SCI_ANNOTATIONGETSTYLES = 2545
SCI_ANNOTATIONGETLINES = 2546
SCI_ANNOTATIONCLEARALL = 2547
ANNOTATION_HIDDEN = 0
ANNOTATION_STANDARD = 1
ANNOTATION_BOXED = 2
SCI_ANNOTATIONSETVISIBLE = 2548
SCI_ANNOTATIONGETVISIBLE = 2549
SCI_ANNOTATIONSETSTYLEOFFSET = 2550
SCI_ANNOTATIONGETSTYLEOFFSET = 2551

SCI_STYLESETCHANGEABLE = 2099
STC_FIND_MATCHCASE = 4
STC_FIND_POSIX = 4194304
STC_FIND_REGEXP = 2097152
STC_FIND_WHOLEWORD = 2
STC_FIND_WORDSTART = 1048576


BOOKMARK_TOGGLE  = 30000
BOOKMARK_UP      = 30001
BOOKMARK_DOWN    = 30002
BREAKPOINT_TOGGLE= 30003
FIND = 30004
FIND_NEXT = 30005
FIND_PREV = 30006


class EditorHandler(EditorHandlerBase):
    """Declares c++ editor handler"""

    keyword_list = [
            "alignas",
            "alignof",
            "and",
            "and_eq",
            "asm",
            "auto",
            "bitand",
            "bitor",
            "break",
            "case",
            "catch",
            "class",
            "compl",
            "const",
            "constexpr",
            "const_cast",
            "continue",
            "decltype",
            "default",
            "delete",
            "do",
            "dynamic_cast",
            "else",
            "enum",
            "explicit",
            "export",
            "extern",
            "false",
            "for",
            "final",
            "friend",
            "goto",
            "if",
            "inline",
            "mutable",
            "namespace",
            "new",
            "noexcept",
            "not",
            "not_eq",
            "nullptr",
            "operator",
            "or",
            "or_eq",
            "private",
            "protected",
            "public",
            "register",
            "reinterpret_cast",
            "return",
            "signed",
            "sizeof",
            "static",
            "static_assert",
            "static_cast",
            "struct",
            "switch",
            "template",
            "this",
            "thread_local",
            "throw",
            "true",
            "try",
            "typedef",
            "typeid",
            "typename",
            "union",
            "unsigned",
            "using",
            "virtual",
            "void",
            "volatile",
            "wchar_t",
            "while",
            "xor xor_eq"
    ]

    style_items = [
        (STC_C_DEFAULT, "default"),
        (STC_C_STRING, "string"),
        (STC_C_STRING + 64, "string (disabled)"),
        (STC_C_PREPROCESSOR, "preprocessor"),
        (STC_C_PREPROCESSOR + 64, "preprocessor (disabled)"),
        (STC_C_IDENTIFIER, "identifier"),
        (STC_C_IDENTIFIER + 64, "identifier (disabled)"),
        (STC_C_NUMBER, "number"),
        (STC_C_NUMBER + 64, "number (disabled)"),
        (STC_C_CHARACTER, "character"),
        (STC_C_CHARACTER + 64, "character (disabled)"),
        (STC_C_WORD, "keyword"),
        (STC_C_WORD + 64, "keyword (disabled)"),
        (STC_C_WORD2, "keyword2"),
        (STC_C_WORD2 + 64, "keyword2 (disabled)"),
        (STC_C_COMMENT, "comment"),
        (STC_C_COMMENT + 64, "comment (disabled)"),
        (STC_C_COMMENTLINE, "line comment"),
        (STC_C_COMMENTLINE + 64, "line comment (disabled)"),
        (STC_C_COMMENTDOC, "comment doc"),
        (STC_C_COMMENTDOC + 64, "comment doc (disabled)"),
        (STC_C_COMMENTDOCKEYWORD, "comment doc keyword"),
        (STC_C_COMMENTDOCKEYWORD + 64, "comment doc keyword (disabled)"),
        (STC_C_COMMENTDOCKEYWORDERROR, "comment doc keyword error"),
        (STC_C_COMMENTDOCKEYWORDERROR + 64, "comment doc keyword error (disabled)"),
        (STC_C_GLOBALCLASS, "global class"),
        (STC_C_GLOBALCLASS + 64, "global class (disabled)"),
        # pike? (STC_C_HASHQUOTEDSTRING, "hash-quoted string"),
        # pike? (STC_C_HASHQUOTEDSTRING + 64, "hash-quoted string (disabled)"),
        (STC_C_OPERATOR, "operator"),
        (STC_C_OPERATOR + 64, "operator (disabled)"),
        (STC_C_PREPROCESSORCOMMENT, "preprocessor comment"),
        (STC_C_PREPROCESSORCOMMENT + 64, "preprocessor comment (disabled)"),
        (STC_C_REGEX, "regex"),
        (STC_C_REGEX + 64, "regex (disabled)"),
        (STC_C_STRINGEOL, "string end-of-line"),
        (STC_C_STRINGEOL + 64, "string end-of-line (disabled)"),
        (STC_C_STRINGRAW, "raw string"),
        (STC_C_STRINGRAW + 64, "raw string (disabled)"),
        (STC_C_VERBATIM, "verbatim string"),
        (STC_C_VERBATIM + 64, "verbatim string (disabled)"),
        (STC_C_UUID, "uuid"),
        (STC_C_UUID + 64, "uuid (disabled)"),
        (STC_STYLE_BRACELIGHT, "brace light"),
        (STC_STYLE_BRACELIGHT + 64, "brace light (inactive)"),
        (STC_STYLE_BRACEBAD, "brace bad"),
        (STC_STYLE_BRACEBAD + 64, "brace bad (inactive)"),
        (SCI_STYLESETCHANGEABLE, "changeable"),
        (STC_STYLE_LINENUMBER, "line numbers"),
        (STC_STYLE_INDENTGUIDE, "ident guide")
        ]

    style = {
            'default': "fore:#FFFFFF, back:#000000",
            'string': "fore:#960000",
            'string (disabled)': "fore:#646464",
            'preprocessor': "fore:#a56900",
            'preprocessor (disabled)': "fore:#646464",
            'identifier': "fore:#28003C",
            'identifier (disabled)': "fore:#646464",
            'number': "fore:#009600",
            'number (disabled)': "fore:#646464",
            'character': "fore:#960000",
            'character (disabled)': "fore:#646464",
            'keyword': "fore:#000096, bold",
            'keyword (disabled)': "fore:#646464, bold",
            'keyword2': "fore:#009600, bold",
            'keyword2 (disabled)': "fore:#646464, bold",
            'comment': "fore:#969696",
            'comment (disabled)': "fore:#646464",
            'line comment': "fore:#969696, back:#FFFFFF",
            'line comment (disabled)': "fore:#646464, back:#FFFFFF",
            'comment doc': "fore:#969696",
            'comment doc (disabled)': "fore:#646464",
            'comment doc keyword': "fore:#0000c8, bold",
            'comment doc keyword (disabled)': "fore:#646464, bold",
            'comment doc keyword error': "fore:#c80000, bold",
            'comment doc keyword error (disabled)': "fore:#646464, bold",
            "brace light": "fore:#000000,back:#7FFF7F,bold",
            "brace light (inactive)": "fore:#000000,back:#7FFF7F,bold",
            "brace bad": "fore:#000000,back:#FF7F7F,bold",
            "brace bad (inactive)": "fore:#000000,back:#FF7F7F,bold",
            "changeable": "back:#5F5F5F,back:#000000,bold",
            "line numbers": "fore:#4B4B4B, back:#DCDCDC",
            "ident guide": "fore:#960000,bold",
        }

    def __init__(self, **kwargs):
        """Init"""
        self._fileObj = kwargs['obj']
        kwargs['lexer'] = STC_LEX_CPP
        kwargs['text'] = self._fileObj.GetText()
        kwargs['read_only'] = self._fileObj._readOnly
        super(EditorHandler, self).__init__(**kwargs)

    def init_fonts(self, editor):
        """default font initialization"""
        super(EditorHandler, self).init_fonts(editor)
        font = self.font
        for i in range(0, 39):
            editor.StyleSetFont(i + 64, font)    # inactive styles

    def SetupMargins(self, editor):
        """Setup margins"""
        #Enable code folding
        editor.SetMarginType(MARK_MARGIN, STC_MARGIN_SYMBOL)
        editor.SetMarginType(MARGIN_LINE_NUMBERS, STC_MARGIN_NUMBER)
        editor.SetMarginType(MARGIN_FOLD, STC_MARGIN_SYMBOL)
        editor.SetMarginWidth(MARK_MARGIN, 11)
        editor.SetMarginWidth(MARGIN_LINE_NUMBERS, 50)
        editor.SetMarginWidth(MARGIN_FOLD, 15)
        editor.SetMarginMask(MARK_MARGIN, STC_MASK_MARKERS)
        editor.SetMarginMask(MARGIN_FOLD, STC_MASK_FOLDERS)
        editor.SetMarginSensitive(MARK_MARGIN, True)
        editor.SetMarginSensitive(MARGIN_FOLD, True)
        editor.SetMarginSensitive(MARGIN_LINE_NUMBERS, True)

    def SetupMarkers(self, editor):
        """Setup markers"""
        editor.MarkerDefineBitmap(MARK_BOOKMARK, wx.BitmapFromXPMData(rc._bookmark))
        editor.MarkerDefine(MARK_BREAKPOINT, STC_MARK_CIRCLE, "blue", "red")
        editor.MarkerDefine(STC_MARKNUM_FOLDEREND, STC_MARK_BOXPLUSCONNECTED, "white", "black")
        editor.MarkerDefine(STC_MARKNUM_FOLDEROPENMID, STC_MARK_BOXMINUSCONNECTED, "white", "black")
        editor.MarkerDefine(STC_MARKNUM_FOLDERMIDTAIL, STC_MARK_TCORNER, "white", "black")
        editor.MarkerDefine(STC_MARKNUM_FOLDERTAIL, STC_MARK_LCORNER, "white", "black")
        editor.MarkerDefine(STC_MARKNUM_FOLDERSUB, STC_MARK_VLINE, "white", "black")
        editor.MarkerDefine(STC_MARKNUM_FOLDER, STC_MARK_BOXPLUS, "white", "black")
        editor.MarkerDefine(STC_MARKNUM_FOLDEROPEN, STC_MARK_BOXMINUS, "white", "black")

    def SetProperties(self, editor):
        """Set other properties"""
        # Properties found from http://www.scintilla.org/SciTEDoc.html
        editor.SetProperty("fold", "1")
        editor.SetProperty("fold.comment", "1")
        editor.SetProperty("fold.compact", "0")
        editor.SetProperty("fold.margin.width", "5")
        editor.SetProperty("indent.automatic", "1")
        editor.SetProperty("indent.opening", "1")
        editor.SetProperty("indent.closing", "1")
        editor.SetProperty("indent.size", "4")
        editor.SetProperty("tabsize", "4")
        editor.SetProperty("indent.size", "4")
        editor.SetProperty("use.tabs", "1")
        editor.SetProperty("tab.indents", "1")
        editor.SetIndent(4)
        editor.SetUseTabs(True)
        editor.SetTabIndents(True)
        editor.SetTabWidth(4)

    def SetupEvents(self, editor):
        """Setup events"""
        editor.CmdKeyAssign(ord('+'), STC_SCMOD_CTRL, STC_CMD_ZOOMIN)
        editor.CmdKeyAssign(ord('-'), STC_SCMOD_CTRL, STC_CMD_ZOOMOUT)
        editor.Bind(wx.EVT_MENU, editor.ToggleBookmark, id=BOOKMARK_TOGGLE)
        editor.Bind(wx.EVT_MENU, editor.PrevBookmark, id=BOOKMARK_UP)
        editor.Bind(wx.EVT_MENU, editor.NextBookmark, id=BOOKMARK_DOWN)
        editor.Bind(wx.EVT_MENU, editor.Find, id=FIND)
        editor.Bind(wx.EVT_MENU, editor.FindNext, id=FIND_NEXT)
        editor.Bind(wx.EVT_MENU, editor.FindPrevious, id=FIND_PREV)
        editor.Bind(EVT_STC_MARGINCLICK, editor.OnMarginClick)
        editor.Bind(wx.EVT_KEY_UP, editor.OnKey)
        editor.Bind(EVT_STC_UPDATEUI, editor.OnUpdateEditUI)
        aTable = wx.AcceleratorTable([
            #wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_F9, BREAKPOINT_TOGGLE), <- mus be handled by the view
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_F2, BOOKMARK_TOGGLE),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, wx.WXK_UP, BOOKMARK_UP),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, wx.WXK_DOWN, BOOKMARK_DOWN),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('F'), FIND),
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_F3, FIND_NEXT),
            wx.AcceleratorEntry(wx.ACCEL_SHIFT, wx.WXK_F3, FIND_PREV),
        ])
        editor.SetAcceleratorTable(aTable)

    def Initialize(self, editor):
        """Base initialization"""
        editor.StyleClearAll()
        editor.SetLexer(self.lexer)
        editor.SetCodePage(STC_CP_UTF8)
        editor.SetStyleBits(editor.GetStyleBitsNeeded())
        self.init_fonts(editor)
        editor.HandleBreakpoints(self._fileObj.breakpoints)
        editor.HandleBookmarks(self._fileObj.bookmarks)
        editor.EnableBreakpoints(True)
        editor.EnableBookmarks(True)
        self.SetProperties(editor)
        self.SetupEvents(editor)
        super(EditorHandler, self).Initialize(editor)


# -*- coding: utf-8 -*-
from BasePane import BasePane
from FullScreen import FullScreen
from NavigatorPane import NavigatorPane
from HelpPreferences import HelpPreferences
from WebPreferences import WebPreferences



# -*- coding: utf-8 -*-
"""
Este fuente implementa el gestor de cooperacion.
Una actividad puede ofrecer operaciones que pueden ser compartidas
con otras actividades, vistas o paneles.
La comparticion de estos recursos se realiza a través de un registro
de operaciones en el que se asocia un nombre unico de operacion a
un conjunto de recursos que pueden ser desde barras de herramientas,
elementos de menu u metodos invocables.

nombre de operacion: los nombres de operacion deben de ser unicos.
Se debe de evitar utilizar nombres que identifiquen al proveedor.

cada operacion registrada constituye un espacio de recursos puestos
a disposicion de los clientes, y que pueden ser utilizados por este
de forma selectiva, mediante la gestion.

"""


class CooperativeResource(object):
    """Implementa un recurso compartido"""

    def __init__(self, name, **kwargs):
        """Inicializa el recurso compartido"""
        self._name = name
        self._data = kwargs
        self._version = kwargs.get('version', [0, 0, 0])
        super(CooperativeResource, self).__init__()

    @property
    def version(self):
        """return the version"""
        return self._version

    @property
    def name(self):
        """return the name"""
        return self._name


class SharedToolbar(CooperativeResource):
    """Implementa una toolbar compartida"""
    def __init__(self, name, **kwargs):
        """Inicializa el recurso compartido"""
        super(SharedToolbar, self).__init__(name, **kwargs)


class Cooperation(object):
    """Declara la estructura de una cooperacion"""

    def __init__(self, name, acm=None):
        """Inicializa la cooperacion"""
        self._name = name
        self._acm = acm  # validador
        self._resource_list = {}
        super(Cooperation, self).__init__()

    def available(self, client):
        """Determina si la cooperacion esta disponible para el cliente"""
        if self._acm is None:
            return True
        if hasattr(self._acm, '__call__'):
            return self._acm(client)
        return bool(self._acm)

    def add(self, resource):
        """Anhade un recurso a la cooperacion"""
        if resource.name in self._resource_list:
            for k in self._resource_list[resource.name]:
                if k.version == resource.version:
                    raise RuntimeError('duplicated resource {name} with version {version}'.format(
                        name=resource.name,
                        version=resource.version))
            self._resource_list[resource.name].append(resource)
        else:
            self._resource_list[resource.name] = [resource]


class CooperativeHandler(object):
    """Implementa el gestor de recursos compartidos.
    Esta clase actua como un singleton, de modo que
    cualquier intento de instanciacion genera un error
    de runtime"""
    _cooperation = {}  # diccionario de cooperaciones registradas
    _key = {}  # claves de acceso a cooperacion

    def __init__(self):
        """Inicializacion: No esta permitida"""
        raise RuntimeError("CooperativeHandler is not instanciable")

    @classmethod
    def register(cls, name, acm=None, key=None):
        """Accede a un registro de cooperacion o lo crea.
        Si el registro de cooperacion no existe, se crea, y se
        le asocia el filtro de acceso suministrado. Si el registro de
        cooperacion ya existe, en caso de que tenga una acm
        no nula asociada, el parametro acm no debe ser nulo
        y debe hacer matching con la clave con la que fue
        registrada la cooperacion. En caso negativo, se genera
        un error de runtime"""
        if name not in cls._cooperation:
            cls._cooperation[name] = Cooperation(name, acm)
            cls._key[name] = key
        elif cls._key[name] != key:
            raise RuntimeError('access to cooperation {name} is not allowed'.format(name=name))
        return cls._cooperation[name]




# -*- coding: utf-8 -*-
import wx


"""Defines handler for identifiers"""


class IdentifiersHandler(object):
    """Application identifiers handler"""
    _identifiers = {}

    def __init__(self):
        """Ensures this is a singleton"""
        raise RuntimeError('IdentifiersHandler is a singleton')
        super(IdentifiersHandler, self).__init__()

    @classmethod
    def new(cls, name=None, value=None):
        """Create a new identifier"""
        if name:
            if name in cls._identifiers:
                raise RuntimeError('Identifier {name} already exists'.format(name=name))
            if value is None:                 
                value = wx.Window.NewControlId()
                while value in cls._identifiers.values():
                    value = wx.Window.NewControlId()
            else:
                assert value not in cls._identifiers.values()
            cls._identifiers[name] = value
            return value
        else:
            return wx.Window.NewControlId()

    @classmethod
    def register(cls, name, value=None):
        """Register a identifier"""
        local_value = cls._identifiers.get(name, None)
        if local_value is None:
            local_value = cls.new(name, value)
        elif value is not None:
            assert value == local_value
        return local_value



# -*- coding: utf-8 -*-

"""
Created on Sun Dec 15 19:22:32 2013

@author: mel
"""

import model
import app.resources as rc
import tran


class Table(model.TComponent):
    """Implements table table representation"""
    folder_container = True
    diagram_container = True
    field_container = True

    # visual methods
    @tran.TransactionalMethod('move table {0}')
    def drop(self, to):
        """Drops schema inside project or another folder """
        return False
        target = to.inner_table_container
        if not target or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization method"""
        super(Table, self).__init__(**kwargs)

    @property
    def query(self):
        return 'SELECT * FROM `{schema}`.`{table}`;'.format(
                schema=self.inner_schema.name,
                table=self.name)

    def CreateFields(self, conn):
        """Create fields for the table from database"""
        try:
            conn.query('DESCRIBE `{schema}`.`{table}`'.format(
                schema=self.inner_schema.name,
                table=self.name))
            data = conn.store_result()
            if data:
                kwargs = {'parent': self}
                for i in range(0, data.num_rows()):
                    info = data.fetch_row()[0]
                    kwargs.update({
                        'name': info[0],
                        'type': info[1],  # 'int(11)'
                        'null': info[2],  # 'YES' or 'NOT'
                        'primary': info[3],  # 'PRI', 'UNI', 'MUL' or ''
                        'default': info[4],  # NULL, '', 0, ...
                        'extra': info[5]  # autoincrement
                        })
                    model.database.Field(**kwargs)
                del data
            return True
        except:
            return False

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs.update(super(Table, self).get_kwargs())
        return kwargs

    def OnUndoRedoRemoving(self):
        """Handle OnUndoRedoRemoving, prevent generating files"""
        super(Table, self).OnUndoRedoRemoving()

    @property
    def can_delete(self):
        """Check abot if class can be deleted"""
        return super(Table, self).can_delete

    def Delete(self):
        """Delete diagram objects"""
        for dia in self.project(model.ClassDiagram):
            # Check if inherit is in
            elem = dia.FindElement(self)
            if elem is not None:
                dia.SaveState()
                dia.RemoveElement(elem)
                if hasattr(dia, '_pane') and dia._pane is not None:
                    dia._pane.Refresh()
        super(Table, self).Delete()

    def OnUndoRedoChanged(self):
        """Update from app"""
        project = self.project
        #update class diagrams
        dias = project(model.ClassDiagram)
        for dia in dias:
            # Check if class is in
            elem = dia.FindElement(self)
            if elem is not None:
                elem.Layout()
                if hasattr(dia, '_pane') and dia._pane is not None:
                    dia._pane.Refresh()
        super(Table, self).OnUndoRedoChanged()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('database_table')

    @property
    def tree_label(self):
        """Get tree label"""
        return self.name

    @property
    def inner_table(self):
        """Get the inner table"""
        return self

    @property
    def outer_table(self):
        """Get the outer class container"""
        return self






from DatabasePane import DatabasePane
from ConstructorPane import ConstructorPane
from DiagramPane import DiagramPane
from MethodPane import MethodPane
from PyMethodPane import PyMethodPane
from PyModulePane import PyModulePane



# -*- coding: utf-8 -*-
from Field import Field
from Table import Table
from Schema import Schema
from DatabaseProject import DatabaseProject
from DatabaseProject import dbopen
from DatabaseProject import rawquery
from DatabaseProject import cursorquery



"""Subclass of HelpPreferences, which is generated by wxFormBuilder."""
import wx
import app
from  ctx import theContext as context

# Implementing HelpPreferences
class HelpPreferences( app.HelpPreferences ):
	def __init__( self, parent ):
		super(HelpPreferences, self).__init__(parent)
		# adjust controls
		sz = self.m_help_items.GetClientSize()
		self.m_help_items.InsertColumn(0, '', wx.LIST_FORMAT_LEFT, sz.x)
		self._changes = False
		self.UpdateData()
		
	def ReadConfig(self):
		"""Read config"""
		self._entry = {}
		self._order = {}
		items = context.config.ReadInt("config/help/items", 3)
		if items >= 1:
			label = context.config.Read("config/help/item/1/label", "cpp reference ...")
			url = context.config.Read("config/help/item/1/url", "http://en.cppreference.com")
			descr = context.config.Read("config/help/item/1/descr", "check online c++ reference")			
			self._entry[label] = (url, descr)
			self._order[1] = label
		if items >= 2:
			label = context.config.Read("config/help/item/2/label", "python 2 reference ...")
			url = context.config.Read("config/help/item/2/url", "https://docs.python.org/2/")
			descr = context.config.Read("config/help/item/2/descr", "check python online reference")			
			self._entry[label] = (url, descr)
			self._order[2] = label
		if items >= 3:
			label = context.config.Read("config/help/item/3/label", "python digger")
			url = context.config.Read("config/help/item/3/url", "http://pydigger.com")
			descr = context.config.Read("config/help/item/3/descr", "")			
			self._entry[label] = (url, descr)
			self._order[3] = label
		#if items >= 4:
		#	 label = context.config.Read("config/help/item/3/label", "online tutorials")
		#	 url = context.config.Read("config/help/item/3/url", "cpp reference ...")
		for index in range(4, items+1):
			label = context.config.Read("config/help/item/{0}/label".format(index), '')
			url = context.config.Read("config/help/item/{0}/url".format(index), '')
			descr = context.config.Read("config/help/item/{0}/descr".format(index), "")			
			if label and url:
				self._entry[label] = (url, descr)
				self._order[index] = label
				
	def SaveConfig(self):
		"""Save config"""
		items = len(self._entry)
		context.config.WriteInt("config/help/items",items)
		for index in range(1, items+1):
			key = self._order[index]
			value, descr = self._entry[key]
			context.config.Write("config/help/item/{0}/label".format(index), key)
			context.config.Write("config/help/item/{0}/url".format(index), value)
			context.config.Write("config/help/item/{0}/descr".format(index), descr)
				
	def UpdateData(self):
		"""Update help preferences data"""
		self.ReadConfig()
		self.m_help_items.DeleteAllItems()
		items = len(self._entry)
		for index in range(1, items+1):
			key = self._order[index]
			#descr = self._entry[key][1]
			self.m_help_items.Append((key,))	
			
	def SaveData(self):		
		"""Save information"""
		if self._changes:
			self.SaveConfig()
			context.frame.BuildHelpMenu()
		
	# Handlers for HelpPreferences events.
	def OnSelectItem( self, event ):
		self.m_edit_btn.Enable(True)
		self.m_del_btn.Enable(True)
		
	def OnDeselectItem(self, event):
		self.m_edit_btn.Enable(False)
		self.m_del_btn.Enable(False)
	
	def OnNewItem( self, event ):
		dlg = app.ui.dlg.HelpItem(self)
		index = len(self._entry)+1
		if dlg.ShowModal() == wx.ID_OK:
			new_key = dlg._label
			new_url = dlg._url
			new_descr = dlg._help
			if new_key in self._entry:
				#alert key already exists
				choice = wx.MessageBox( "The item {0} already exists.\nOwerwrite?".format(new_key),
					"Warning", wx.YES_NO | wx.ICON_INFORMATION, self)
				if choice is not wx.YES:
					return
				index = dict([(y,x) for (x,y) in self._order])[new_key]				
			else:
				self.m_help_items.Append((new_key,))	
			self._changes = True
			self._entry[new_key] = (new_url,new_descr)
			self._order[index] = new_key 
			
	def OnEnterItem(self, event):
		self.OnEditItem(event)
	
	def OnEditItem( self, event ):
		# TODO: Implement OnEditItem
		index = self.m_help_items.GetFirstSelected()
		if index is wx.NOT_FOUND:
			return
		key = self.m_help_items.GetItemText(index)
		if key not in self._entry:
			return
		url, descr = self._entry[key]
		dlg = app.ui.dlg.HelpItem(self)
		dlg.SetAttributes(key, url, descr)
		if dlg.ShowModal() == wx.ID_OK:
			new_key = dlg._label
			new_url = dlg._url
			new_descr = dlg._help
			if new_key != key:
				self._changes = True
				del self._entry[key]
				self.m_help_items.SetItemText(index, new_key)
				self._entry[new_key] = (new_url, new_descr)
				self._order[index+1] = new_key
			elif new_url != url or new_descr != descr:
				self._changes = True
				self._entry[new_key] = (new_url, new_descr)
	
	def OnDeleteItem( self, event ):
		index = self.m_help_items.GetFirstSelected()
		if index is not wx.NOT_FOUND:
			key = self.m_help_items.GetItemText(index)
			if key in self._entry:
				index = index + 1
				del self._entry[key]
				del self._order[index]
				index = index + 1
				for k in range(index, len(self._entry)+2): 
					self._order[k] = k -1
				self._changes = True
			self.m_help_items.DeleteItem(index)
		self.m_edit_btn.Enable(False)
		self.m_del_btn.Enable(False)
		
	
	
	



# -*- coding: utf-8 -*-

"""Subclass of PyModulePane, which is generated by wxFormBuilder."""

#import wx
import wx, wx.stc, wxx, app
from ctx import theContext as context
import tran
import pane


# Implementing PyModulePane
class NavigatorPane(app.NavigatorPane):
    """Implements navigator pane"""
    def __init__(self, parent, mainframe):
        """Intialization of method editor"""
        super(NavigatorPane, self).__init__(parent)
        self._mainframe = mainframe
        self._url = None
        self.Bind(wx.html2.EVT_WEBVIEW_LOADED, self.OnPageLoaded, self.m_page)
        self.use_proxy=context.config.ReadBool("config/web/useProxy", False)
        if self.use_proxy:
            import platform
            s_plat = platform.platform() 
            if 'Linux' in s_plat or 'linux' in s_plat:
                import os
                import ctypes
                http_proxy=context.config.Read("config/web/http_proxy",'')
                same=context.config.ReadBool("config/web/same_proxy",False)
                if not same:
                    # This is not yet implemented. We need a GSimpleProxyResolver
                    # instead of setting a proxy-uri
                    https_proxy=context._config.Read("config/web/https_proxy",'')
                    ftp_proxy=context._config.Read("config/web/ftp_proxy",'')                    
                try:
                    self.soup = ctypes.cdll.LoadLibrary('libsoup-2.4.so')
                    self.webkit = ctypes.cdll.LoadLibrary('libwebkitgtk-1.0.so')
                    self.gobject = ctypes.cdll.LoadLibrary('libgobject-2.0.so')
                    if '64bit' in platform.architecture():
                        self.webkit.webkit_get_default_session.restype = ctypes.c_uint64
                        self.soup.soup_uri_new.restype = ctypes.c_uint64
                        self.gobject.g_object_set.argtypes=(ctypes.c_uint64, ctypes.c_char_p, ctypes.c_uint64, ctypes.c_uint64)
                        self.soup.soup_uri_free.argtypes=(ctypes.c_uint64,)
                    else:
                        #untested
                        self.webkit.webkit_get_default_session.restype = ctypes.c_uint32
                        self.soup.soup_uri_new.restype = ctypes.c_uint32
                        self.gobject.g_object_set.argtypes=(ctypes.c_uint32, ctypes.c_char_p, ctypes.c_uint32, ctypes.c_uint32)
                        self.soup.soup_uri_free.argtypes=(ctypes.c_uint32,)
                    proxy_uri = self.soup.soup_uri_new('http://{proxy}'.format(proxy=http_proxy))
                    self.session = self.webkit.webkit_get_default_session()
                    self.gobject.g_object_set(self.session, "proxy-uri", proxy_uri, 0)
                    self.soup.soup_uri_free(proxy_uri)
                except Exception as e:
                    wx.MessageBox('Failed to setup a proxy','Error',
                        wx.OK | wx.CENTER | wx.ICON_ERROR, self)

    def PreDelete(self):
        """Remove toolbar first in order to avoid gtk-collision at close"""
        #self.m_auiToolBar5.Destroy()

    def NavigateTo(self, str_url):
        """Do navigation"""
        self._url = str_url
        self.m_url.Clear()
        self.m_url.WriteText(str_url)
        self.m_page.LoadURL(str_url)

    def OnPageLoaded(self, event):
        """The page is fully loaded"""
        #self.m_page.RunScript('<..some javascript code>')






# -*- coding: utf-8 -*-
import wx
from wx.stc import *
from  ctx import theContext as context


"""Defines basic interface for app/editor"""


class EditorHandlerBase(object):
    """Interface for editor handlers"""

    cookie_lexers = {
        'text': STC_LEX_NULL,
        'c++': STC_LEX_CPP,
        'python': STC_LEX_PYTHON,
        'sql': STC_LEX_SQL,
        'bash': STC_LEX_BASH,
        'css': STC_LEX_CSS,
        'html': STC_LEX_HTML,
        'mathlab': STC_LEX_MATLAB,
        'octave': STC_LEX_OCTAVE,
        'xml': STC_LEX_XML,
        'apache': STC_LEX_CONF,
        'make': STC_LEX_MAKEFILE,
        'nasm': STC_LEX_ASM,
        'vhdl': STC_LEX_VHDL
    }

    keyword_list = []
    keyword_list2 = []
    style_items = []
    style = {}

    def __init__(self, **kwargs):
        """Initialize handler"""
        self.lexer = kwargs.get('lexer', 'text')
        self.font = kwargs.get('font', 'config/defaultFont')
        self.text = kwargs.get('text', '')
        if 'keywords' in kwargs:
            self.keyword_list = kwargs['keywords']
        if 'keywords2' in kwargs:
            self.keyword_list2 = kwargs['keywords2']
        self.read_only = kwargs.get('read_only', False)
        super(EditorHandlerBase, self).__init__()

    @property
    def lexer(self):
        """get property"""
        return self._lexer

    @lexer.setter
    def lexer(self, value):
        """set property"""
        if value in self.cookie_lexers:
            self._lexer = self.cookie_lexers[value]
        else:
            self._lexer = value

    @property
    def font(self):
        """return the font"""
        return self._font

    @font.setter
    def font(self, font_section):
        """set the font by config section. fallback to default"""
        self._font = wx.SystemSettings_GetFont(wx.SYS_DEFAULT_GUI_FONT)
        font_info = self._font.GetNativeFontInfo().ToString()
        font_info = context.config.Read(font_section, font_info)
        self._font.SetNativeFontInfoFromString(font_info)

    @property
    def keywords(self):
        """return keywords"""
        return ' '.join(self.keyword_list)

    @property
    def keywords2(self):
        """return keywords"""
        return ' '.join(self.keyword_list2)

    def SetupMargins(self, editor):
        """Setup margins"""
        pass

    def SetupMarkers(self, editor):
        """Setup markers"""
        pass

    def SetProperties(self, editor):
        """Set other properties"""
        pass

    def SetupEvents(self, editor):
        """Set event handling"""
        pass

    def init_fonts(self, editor):
        """default font initialization"""
        font = self.font
        for i in range(0, 39):
            editor.StyleSetFont(i, font)

    def InitStyles(self, editor):
        """Initialize the styles"""
        for item in self.style_items:
            (identifier, name) = item
            if name in self.style:
                editor.StyleSetSpec(identifier, self.style[name])

    def InitKeywords(self, editor):
        """Initialize keywords"""
        editor.SetKeyWords(0, self.keywords)
        editor.SetKeyWords(1, self.keywords2)

    def SetupAuto(self, editor):
        """Setup autosuggested"""
        s = self.keyword_list + self.keyword_list2
        s.sort()
        editor._auto = ' '.join(s)

    def Initialize(self, editor):
        """Initialize editor"""
        #if self.keywords:
        #    editor.SetKeyWords(0, self.keywords)
        self.SetupMargins(editor)
        self.SetupMarkers(editor)
        self.SetProperties(editor)
        self.InitStyles(editor)
        self.InitKeywords(editor)
        self.SetupAuto(editor)
        editor.SetText(self.text)
        self.SetupEvents(editor)
        editor.EmptyUndoBuffer()
        editor.SetSavePoint()
        editor.SetUndoCollection(True)
        editor.SetReadOnly(self.read_only)




# -*- coding: utf-8 -*-
import app.resources as rc
import model


class dbopen(object):
    """controller"""

    def __init__(self, database):
        """"""
        self._database = database
        super(dbopen, self).__init__()

    def __enter__(self):
        """Enter operation"""
        self.conn = self._database.open()
        return self.conn

    def __exit__(self, type, value, traceback):
        """"""
        if type is None:
            self._database.close(self.conn)
            return True
        return False


class rawquery(object):
    """controller"""
    def __init__(self, conn, sql):
        """"""
        self._conn = conn
        self._sql = sql
        super(rawquery, self).__init__()

    def __enter__(self):
        """"""
        self._conn.query(self._sql)
        self._data = self._conn.store_result()
        return self._data

    def __exit__(self, type, value, traceback):
        """"""
        if type is None:
            del self._data
            return True
        return False


class cursorquery(object):
    """controller"""
    def __init__(self, conn, sql):
        """"""
        self._conn = conn
        self._sql = sql
        super(cursorquery, self).__init__()

    def __enter__(self):
        """"""
        self._cur = self._conn.cursor()
        self._cur.execute(self._sql)
        return self._cur

    def __exit__(self, type, value, traceback):
        """"""
        if type is None:
            self._cur.close()
        return False



## From now, we start to handle different project flawors from handlers
class DatabaseProject(object):
    """Handles a database project"""

    def __init__(self, owner, **kwargs):
        """Initialize database project"""
        self._owner = owner
        self._database_host = kwargs.get('database_host', 'localhost')
        self._database_user = kwargs.get('database_user', 'root')
        self._database_password = kwargs.get('database_password', '')
        self._database_default = kwargs.get('database_default', '')
        super(DatabaseProject, self).__init__()
        # Update database schemas

    def Initialize(self):
        """Create first time. It's called from project container ctor'"""
        self._owner.schema_container = True
        self.CreateSchemas()
        self._owner.CreateTasks()

    def open(self):
        """Open connection"""
        import MySQLdb as driver
        return driver.connect(
            host=self._database_host,
            user=self._database_user,
            passwd=self._database_password)

    def close(self, conn):
        """close connection"""
        conn.close()

    def CreateSchemas(self):
        """Attempt to recover schemas from database and create it"""
        try:
            with dbopen(self) as conn:
                with rawquery(conn, 'SHOW SCHEMAS') as data:
                    if data:
                        kwargs = {'parent': self._owner}
                        for i in range(0, data.num_rows()):
                            kwargs['name'] = data.fetch_row()[0][0]
                            model.database.Schema(**kwargs)
                #Ok now we iterate over schemas and create tables
                for schema in self._owner[model.database.Schema]:
                    schema.CreateTables(conn)
                return True
        except:
            return False

    def get_kwargs(self):
        """get construction data"""
        return {
            'database_host': self._database_host,
            'database_user': self._database_user,
            'database_password': self._database_password,
            'database_default': self._database_default
        }

    def GetTabBitmap(self):
        """return tab bitmap"""
        return rc.GetBitmap('databases')

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('databases')





"""Subclass of NewHelpItem, which is generated by wxFormBuilder."""

import wx
import wxx
import app

# Implementing NewHelpItem
class HelpItem( app.NewHelpItem ):
	"""
	This dialog allows to setup a help item.
	Help items are displayed in help menu.
	"""
	@wxx.SetInfo(__doc__)
	def __init__( self, parent ):
		super(HelpItem, self).__init__(parent)
	
	# Handlers for NewHelpItem events.
	def OnOK( self, event ):
		self._label = self.m_textCtrl56.GetValue()
		self._url = self.m_textCtrl29.GetValue()
		self._help = self.m_textCtrl30.GetValue()
		self.EndModal(wx.ID_OK)
			
	def SetAttributes(self, label, url, help):
		"""Transfer attributes from existing constructor"""
		self._label = label
		self._url = url
		self._help = help
		self.m_textCtrl56.SetValue(self._label)
		self.m_textCtrl29.SetValue(self._url)
		self.m_textCtrl30.SetValue(self._help)
		self.SetTitle("Edit help item")



# -*- coding: utf-8 -*-

"""
Created on Sun Dec 15 19:22:32 2013

@author: mel
"""

import model
import app.resources as rc
import tran


class Field(model.TComponent):
    """Implements field table representation"""

    # visual methods
    @tran.TransactionalMethod('move field {0}')
    def drop(self, to):
        """Drops schema inside project or another folder """
        return False
        target = to.inner_field_container
        if not target or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization method"""
        self._type = kwargs.get('type', None)
        self._null = kwargs.get('null', 'YES')
        self._primary = kwargs.get('primary', '')
        self._default = kwargs.get('default', None)
        self._extra = kwargs.get('extra', '')
        super(Field, self).__init__(**kwargs)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {
            'type': self._type,
            'null': self._null,
            'primary': self._primary,
            'default': self._default,
            'extra': self._extra
        }
        kwargs.update(super(Field, self).get_kwargs())
        return kwargs

    def OnUndoRedoRemoving(self):
        """Handle OnUndoRedoRemoving, prevent generating files"""
        super(Field, self).OnUndoRedoRemoving()

    @property
    def can_delete(self):
        """Check abot if class can be deleted"""
        return super(Field, self).can_delete

    def Delete(self):
        """Delete diagram objects"""
        for dia in self.project(model.ClassDiagram):
            # Check if inherit is in
            elem = dia.FindElement(self)
            if elem is not None:
                dia.SaveState()
                dia.RemoveElement(elem)
                if hasattr(dia, '_pane') and dia._pane is not None:
                    dia._pane.Refresh()
        super(Field, self).Delete()

    def OnUndoRedoChanged(self):
        """Update from app"""
        project = self.project
        #update class diagrams
        dias = project(model.ClassDiagram)
        for dia in dias:
            # Check if class is in
            elem = dia.FindElement(self)
            if elem is not None:
                elem.Layout()
                if hasattr(dia, '_pane') and dia._pane is not None:
                    dia._pane.Refresh()
        super(Field, self).OnUndoRedoChanged()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('database_field')

    @property
    def tree_label(self):
        """Get tree label"""
        return self.name

    @property
    def inner_field(self):
        """Get the inner field"""
        return self

    @property
    def outer_field(self):
        """Get the outer class container"""
        return self






"""Subclass of Preferences, which is generated by wxFormBuilder."""

import wx, wxx
import app
from  app.ui import pane
from  ctx import theContext as context


# Implementing Preferences
class PreferencesDialog(app.Preferences):
    """
    This dialog is used for setup of common preferences.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent):
        """Initialization"""
        super(PreferencesDialog, self).__init__(parent)
        self._config = context.config
        self._fontPane = app.FontPreferences(self.m_auinotebook3)
        self._helpPane = pane.HelpPreferences(self.m_auinotebook3)
        import platform
        s_plat = platform.platform() 
        if 'Linux' in s_plat or 'linux' in s_plat:
            self._linux = True
        else:
            self._linux = False
        if self._linux:
            self._webPane  = pane.WebPreferences(self.m_auinotebook3,
                use_proxy=self._config.ReadBool("config/web/useProxy", False),
                same=self._config.ReadBool("config/web/same_proxy",False),
                http_proxy=self._config.Read("config/web/http_proxy",''),
                https_proxy=self._config.Read("config/web/https_proxy",''),
                ftp_proxy=self._config.Read("config/web/ftp_proxy",''))
        if wx.__version__ >= '3.0.0.0':
            self.m_auinotebook3.AddPage(self._fontPane, u"Fonts", False, wx.wx.NOT_FOUND)
            self.m_auinotebook3.AddPage(self._helpPane, u"Help", False, wx.wx.NOT_FOUND)
            if self._linux:
                self.m_auinotebook3.AddPage(self._webPane, u"Web", False, wx.wx.NOT_FOUND)
        else:
            self.m_auinotebook3.AddPage(self._fontPane, u"Fonts", False, wx.NullBitmap)
            self.m_auinotebook3.AddPage(self._helpPane, u"Help", False, wx.NullBitmap)
            if self._linux:
                self.m_auinotebook3.AddPage(self._webPane, u"Web", False, wx.NullBitmap)
        df = wx.SystemSettings_GetFont(wx.SYS_DEFAULT_GUI_FONT)
        dfs = self.GetFontString(df)
        dfs = self._config.Read("config/defaultFont", dfs)
        self._font = self.GetStringFont(dfs)
        

    def GetFontString(self, font):
        """Get info about current font"""
        return font.GetNativeFontInfo().ToString()

    def GetStringFont(self, string):
        """Get info current font"""
        f = wx.SystemSettings_GetFont(wx.SYS_DEFAULT_GUI_FONT)
        f.SetNativeFontInfoFromString(string)
        return f

    # Handlers for Preferences events.
    def OnInitDialog(self, event):
        """Dialog initialization"""
        self._fontPane.m_fontPicker.SetSelectedFont(self._font)
        # TODO: Implement OnInitDialog
        pass

    def OnOK(self, event):
        """OnOK handler"""
        dfs = self.GetFontString(self._fontPane.m_fontPicker.GetSelectedFont())
        self._webPane.GetData()
        self._config.Write("config/defaultFont", dfs)
        if self._linux:
            self._config.WriteBool("config/web/useProxy",self._webPane._use_proxy)
            self._config.WriteBool("config/web/same_proxy",self._webPane._use_same)
            self._config.Write("config/web/http_proxy",self._webPane._http_proxy.encode('UTF-8'))
            self._config.Write("config/web/https_proxy",self._webPane._https_proxy.encode('UTF-8'))
            self._config.Write("config/web/ftp_proxy",self._webPane._ftp_proxy.encode('UTF-8'))
        self._helpPane.SaveData()
        self._config.Flush()
        self.EndModal(wx.ID_OK)





"""Subclass of NewProject, which is generated by wxFormBuilder."""

import os, wx, wxx, app
import app.resources as rc
from ctx import localcontext as context


# Implementing NewProject
class ProjectDialog(app.NewProject):
    """
    This dialog is the start point for a new project to
    be created. You  may select the target language and
    set his common options. The language may be set only
    before the project is created.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container=None):
        """Initialization"""
        super(ProjectDialog, self).__init__(parent)
        stdPaths = wx.StandardPaths.Get()
        self._container = container
        # if this project is inside workspace, the base
        # path must be determined by this workspace
        wk = container.workspace
        if wk is not None:
            self._base = wk._dir
            self.m_bpButton3.Enable(False)
            self.m_textCtrl9.SetEditable(False)
            self.m_bpButton3.Enable(False)
        else:
            self._base = stdPaths.GetDocumentsDir()
        self.m_textCtrl9.SetValue(self._base)
        self.app = context.app
        self._original = ""
        self._autodir = True
        self._veto = False
        self.m_textCtrl1.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap('cppproject'))
        self.SetIcon(icon)
        self._database_host = ''
        self._database_user = ''
        self._database_password = ''

    def CopyAttributes(self, prj):
        """Update project attributes with dialog values"""
        sel = self.m_choicebook1.GetSelection()
        prj._name = self._name
        if sel == 0:
            prj._language = 'c++'
        elif sel == 1:
            prj._language = 'python'
        elif sel == 2:
            prj._language = 'databases'
        prj._dir = self._dir
        if sel in [0, 1]:
            prj._includeDir = self.m_textCtrl23.GetValue()
            prj._srcDir = self.m_textCtrl24.GetValue()
            prj._useMaster = self.m_checkBox52.GetValue()
            prj._masterInclude = self.m_textCtrl25.GetValue()
            prj._author = self.m_textCtrl251.GetValue()
            prj._date = self.m_datePicker1.GetValue().FormatISODate()
            if self.m_checkBox53.IsChecked():
                prj._license = self.m_choice16.GetStringSelection()
            else:
                prj._license = None
            prj._type = self.m_choice151.GetStringSelection()
            prj._version[0] = self.m_spinCtrl11.GetValue()
            prj._version[1] = self.m_spinCtrl21.GetValue()
            prj._version[2] = self.m_spinCtrl31.GetValue()
            prj._useMakefile = self.m_checkBox54.IsChecked()
            prj.note = self.m_richText13.GetValue()
        else:
            handler = prj._handler
            handler._database_host = self._database_host
            handler._database_user = self._database_user
            handler._database_password = self._database_password
            handler._database_default = self._database_default

    def SetAttributes(self, prj):
        """Set dialog attributes based on existing project"""
        if prj._language == 'c++':
            self.m_choicebook1.SetSelection(0)
        elif prj._language == 'python':
            self.m_choicebook1.SetSelection(1)
        elif prj._language == 'databases':
            self.m_choicebook1.SetSelection(2)
        self._veto = True
        self.m_textCtrl1.SetValue(prj._name)
        self.m_textCtrl9.SetValue(prj._dir)
        if prj._language in ['c++', 'python']:
            self.m_textCtrl23.SetValue(prj._includeDir)
            self.m_textCtrl24.SetValue(prj._srcDir)
            self.m_checkBox52.SetValue(prj._useMaster)
            self.m_textCtrl25.Enable(prj._useMaster)
            self.m_textCtrl25.SetValue(prj._masterInclude)
            self.m_textCtrl251.SetValue(prj._author)
            date = wx.DateTime()
            date.ParseDate(prj._date)
            self.m_datePicker1.SetValue(date)
            if prj._license is not None:
                self.m_checkBox53.SetValue(True)
                self.m_choice16.SetStringSelection(prj._license)
            else:
                self.m_checkBox53.SetValue(False)
            self.m_choice151.SetStringSelection(prj._type)
            self.m_spinCtrl11.SetValue(prj._version[0])
            self.m_spinCtrl21.SetValue(prj._version[1])
            self.m_spinCtrl31.SetValue(prj._version[2])
            self.m_checkBox54.SetValue(prj._useMakefile)
            self.m_richText13.SetValue(prj.note)
        elif prj._language == 'databases':
            handler = prj._handler
            self.m_textCtrl252.SetValue(handler._database_host)
            self.m_textCtrl231.SetValue(handler._database_user)
            self.m_textCtrl241.SetValue(handler._database_password)
            self.m_comboBox1.SetValue(handler._database_default)
        self._original = prj._name
        self.SetTitle("Edit project")

    def OnPageChanged(self, event):
        """Show/Hide some controls from project type"""
        sel = event.GetSelection()
        icon = wx.EmptyIcon()
        if  sel == 2:
            # This is a database project
            self.m_panel6.Hide()
            self.m_panel7.Hide()
            icon.CopyFromBitmap(rc.GetBitmap('databases'))
        else:
            self.m_panel6.Show()
            self.m_panel7.Show()
            if sel == 0:
                icon.CopyFromBitmap(rc.GetBitmap('cppproject'))
            elif sel == 1:
                icon.CopyFromBitmap(rc.GetBitmap('python'))
        self.fgSizer4.Layout()
        self.fgSizer65.Layout()
        self.SetIcon(icon)

    def OnPageChanging(self, event):
        """Checks if project type can be changed"""
        if self._veto:
            event.Veto()
        else:
            event.Skip()

    def OnChangeProjectName(self, event):
        """Process change text event"""
        if self._autodir:
            name = self.m_textCtrl1.GetValue()
            #create a custom subdirectory
            self.m_textCtrl9.SetValue(os.path.join(self._base, name))
            if self.m_checkBox52.GetValue():
                #replace master include
                name = name.replace(" ", "_")
                name += ".h"
                self.m_textCtrl25.SetValue(name)

    def OnChooseDir(self, event):
        """Launch subdialog for choosing dir"""
        dialog = wx.DirDialog(self, "Select directory",
            self.m_textCtrl9.GetValue(), wx.DD_DIR_MUST_EXIST)
        if dialog.ShowModal() == wx.ID_OK:
            self.m_textCtrl9.SetValue(dialog.GetPath())
            self._autodir = False

    def OnToggleMasterInclude(self, event):
        """Toggle using master include file"""
        if self.m_checkBox52.GetValue():
            self.m_textCtrl25.Enable(True)
            self.m_textCtrl25.SetValue(self.m_textCtrl1.GetValue() + ".h")
        else:
            self.m_textCtrl25.Enable(False)

    # Database project handlers
    def on_mysql_host_change(self, event):
        """Handle database host change"""
        self._database_host = self.m_textCtrl252.GetValue()
        self.UpdateTestDatabaseConnection()

    def on_mysql_user_change(self, event):
        """Handle user host change"""
        self._database_user = self.m_textCtrl231.GetValue()
        self.UpdateTestDatabaseConnection()

    def on_mysql_password_change(self, event):
        """Handle user password change"""
        self._database_password = self.m_textCtrl241.GetValue()
        self.UpdateTestDatabaseConnection()

    def UpdateTestDatabaseConnection(self):
        """Update the enabled/disabled status of the test connection button"""
        if len(self._database_host) > 0 and len(self._database_user) > 0 and len(self._database_password) > 0:
            self.m_button3.Enable(True)
        else:
            self.m_button3.Enable(False)

    def OnTestDatabaseConnection(self, event):
        """Test the database connection"""
        if self.TestDatabaseConnection():
            self.UpdateSchemas()

    def GetMySQLdbDriver(self):
        """Attempt to import MySQLdb and return status"""
        try:
            import MySQLdb
        except:
            return False
        return MySQLdb

    def TestDatabaseConnection(self, verbose=True):
        """Check the database connectivity"""
        retry = False
        driver = self.GetMySQLdbDriver()
        if not driver:
            reponse = wx.MessageBox("\nThe mysql-python package is not installed.\n\n"
                "You need have installed python-dev and libmysqlclient-dev \n"
                "and mysql-python package. For example doing:\n\n"
                " > sudo apt-get install python-dev libmysqlclient-dev \n"
                " > (sudo) pip install mysql-python.\n\n"
                "Do you want we attempt to install mysql-python for you?",
                "Error", wx.YES_NO | wx.CENTER | wx.ICON_ERROR, context.frame)
            if reponse != wx.YES:
                return
            if os.system("gksudo 'apt-get -y install python-dev libmysqlclient-dev'"):
                wx.MessageBox("Install failed!", "Error", wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
                return
            if os.system("pip install mysql-python"):
                #Ok, now with sudo
                if os.system("gksudo 'pip install mysql-python'"):
                    wx.MessageBox("Install failed!", "Error", wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
                    return
            retry = True
        if retry:
            driver = self.GetMySQLdbDriver()
            if driver:
                wx.MessageBox("The install seems ok, but import mysql-python still fails.\n"
                "Try to close and reope beatle.\n"
                "Sorry for the inconvenience", "Error", wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
                return
        try:
            conn = driver.connect(host=self._database_host,
                        user=self._database_user,
                        passwd=self._database_password)
        except driver.Error:
            wx.MessageBox("Connection failed.", "Error", wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
            return False
        conn.close()
        if verbose:
            wx.MessageBox("Connection success.", "Error", wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
        return True

    def UpdateSchemas(self):
        """Update the schema list"""
        driver = self.GetMySQLdbDriver()
        conn = driver.connect(host=self._database_host, user=self._database_user, passwd=self._database_password)
        conn.query('SHOW SCHEMAS')
        data = conn.store_result()
        schemas = []
        for i in range(0, data.num_rows()):
            schemas.append(data.fetch_row()[0][0])
        del data
        conn.close()
        self.m_comboBox1.Clear()
        self.m_comboBox1.AppendItems(schemas)

    def Validate(self):
        """Validate dialog"""
        sel = self.m_choicebook1.GetSelection()
        if sel == 0:
            self._language = 'c++'
        elif sel == 1:
            self._language = 'python'
        elif sel == 2:
            self._language = 'databases'
        self._name = self.m_textCtrl1.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Project name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if not self._original == self._name:
            if self.app.ExistProject(self._name) is True:
                wx.MessageBox("Project already exist", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        self._dir = self.m_textCtrl9.GetValue()
        if len(self._dir) == 0:
            wx.MessageBox("Target directory must be non empty",
                "Error", wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if sel in [0, 1]:
            # python and c++
            if self.m_checkBox52.GetValue():
                if len(self.m_textCtrl25.GetValue()) == 0:
                    wx.MessageBox("Master include must be non empty",
                        "Error", wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                    return False
            self._includeDir = self.m_textCtrl23.GetValue()
            self._srcDir = self.m_textCtrl24.GetValue()
            self._useMaster = self.m_checkBox52.GetValue()
            self._masterInclude = self.m_textCtrl25.GetValue()
            self._author = self.m_textCtrl251.GetValue()
            self._date = self.m_datePicker1.GetValue().FormatISODate()
            self._license = self.m_choice16.GetStringSelection()
            self._type = self.m_choice151.GetStringSelection()
            self._version = []
            self._version = [
                self.m_spinCtrl11.GetValue(),
                self.m_spinCtrl21.GetValue(),
                self.m_spinCtrl31.GetValue()]
            self._useMakefile = True
        else:
            self._database_host = self.m_textCtrl252.GetValue()
            self._database_user = self.m_textCtrl231.GetValue()
            self._database_password = self.m_textCtrl241.GetValue()
            self._database_default = self.m_comboBox1.GetValue()
            if not self.TestDatabaseConnection(False):
                return False
        return True

    def get_kwargs(self):
        """returns arguments for instance... pending"""
        kwargs = {
            'parent': self._container,
            'language': self._language,
            'name': self._name,
            'dir': self._dir,
        }
        if self._language in ['c++', 'python']:
            kwargs.update({
                'includedir': self._includeDir,
                'sourcedir': self._srcDir,
                'usemaster': self._useMaster,
                'masterinclude': self._masterInclude,
                'author': self._author,
                'date': self._date,
                'license': self._license,
                'type': self._type,
                'version': self._version,
                'usemakefile': self._useMakefile
            })
        elif self._language == 'databases':
            kwargs.update({
                'database_host': self._database_host,
                'database_user': self._database_user,
                'database_password': self._database_password,
                'database_default': self._database_default,
            })
        return kwargs

    # Handlers for NewProject events.
    def OnOK(self, event):
        """Handle ok button"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





# -*- coding: utf-8 -*-
import wx
from wx.stc import *
from app.handlers import EditorHandlerBase
import app.resources as rc

MARGIN_LINE_NUMBERS = 0
MARK_MARGIN = 1
MARGIN_FOLD = 2

STC_MASK_MARKERS = ~STC_MASK_FOLDERS

MARK_BOOKMARK = 0
MARK_BREAKPOINT = 1
DEBUG_CURRENT = 32

#missing indicators
STC_INDIC_FULLBOX = 16

#undefined wrappers
SCI_ANNOTATIONSETTEXT = 2540
SCI_ANNOTATIONGETTEXT = 2541
SCI_ANNOTATIONSETSTYLE = 2542
SCI_ANNOTATIONGETSTYLE = 2543
SCI_ANNOTATIONSETSTYLES = 2544
SCI_ANNOTATIONGETSTYLES = 2545
SCI_ANNOTATIONGETLINES = 2546
SCI_ANNOTATIONCLEARALL = 2547
ANNOTATION_HIDDEN = 0
ANNOTATION_STANDARD = 1
ANNOTATION_BOXED = 2
SCI_ANNOTATIONSETVISIBLE = 2548
SCI_ANNOTATIONGETVISIBLE = 2549
SCI_ANNOTATIONSETSTYLEOFFSET = 2550
SCI_ANNOTATIONGETSTYLEOFFSET = 2551

SCI_STYLESETCHANGEABLE = 2099
STC_FIND_MATCHCASE = 4
STC_FIND_POSIX = 4194304
STC_FIND_REGEXP = 2097152
STC_FIND_WHOLEWORD = 2
STC_FIND_WORDSTART = 1048576


BOOKMARK_TOGGLE  = 30000
BOOKMARK_UP      = 30001
BOOKMARK_DOWN    = 30002
BREAKPOINT_TOGGLE= 30003
FIND = 30004
FIND_NEXT = 30005
FIND_PREV = 30006


class EditorHandler(EditorHandlerBase):
    """Declares SQL editor handler"""

    keyword_list = [
        "ACCESSIBLE",
        "ACCOUNT",
        "ACTION",
        "ADD",
        "AFTER",
        "AGAINST",
        "AGGREGATE",
        "ALGORITHM",
        "ALL",
        "ALTER",
        "ALWAYS",
        "ANALYSE",
        "ANALYZE",
        "AND",
        "ANY",
        "AS",
        "ASC",
        "ASCII",
        "ASENSITIVE",
        "AT",
        "AUTOEXTEND_SIZE",
        "AUTO_INCREMENT",
        "AVG",
        "AVG_ROW_LENGTH",
        "BACKUP",
        "BEFORE",
        "BEGIN",
        "BETWEEN",
        "BIGINT",
        "BINARY",
        "BINLOG",
        "BIT",
        "BLOB",
        "BLOCK",
        "BOOL",
        "BOOLEAN",
        "BOTH",
        "BTREE",
        "BY",
        "BYTE",
        "CACHE",
        "CALL",
        "CASCADE",
        "CASCADED",
        "CASE",
        "CATALOG_NAME",
        "CHAIN",
        "CHANGE",
        "CHANGED",
        "CHANNEL",
        "CHAR",
        "CHARACTER",
        "CHARSET",
        "CHECK",
        "CHECKSUM",
        "CIPHER",
        "CLASS_ORIGIN",
        "CLIENT",
        "CLOSE",
        "COALESCE",
        "CODE",
        "COLLATE",
        "COLLATION",
        "COLUMN",
        "COLUMNS",
        "COLUMN_FORMAT",
        "COLUMN_NAME",
        "COMMENT",
        "COMMIT",
        "COMMITTED",
        "COMPACT",
        "COMPLETION",
        "COMPRESSED",
        "COMPRESSION",
        "CONCURRENT",
        "CONDITION",
        "CONNECTION",
        "CONSISTENT",
        "CONSTRAINT",
        "CONSTRAINT_CATALOG",
        "CONSTRAINT_NAME",
        "CONSTRAINT_SCHEMA",
        "CONTAINS",
        "CONTEXT",
        "CONTINUE",
        "CONVERT",
        "CPU",
        "CREATE",
        "CROSS",
        "CUBE",
        "CURRENT",
        "CURRENT_DATE",
        "CURRENT_TIME",
        "CURRENT_TIMESTAMP",
        "CURRENT_USER",
        "CURSOR",
        "CURSOR_NAME",
        "DATA",
        "DATABASE",
        "DATABASES",
        "DATAFILE",
        "DATE",
        "DATETIME",
        "DAY",
        "DAY_HOUR",
        "DAY_MICROSECOND",
        "DAY_MINUTE",
        "DAY_SECOND",
        "DEALLOCATE",
        "DEC",
        "DECIMAL",
        "DECLARE",
        "DEFAULT",
        "DEFAULT_AUTH",
        "DEFINER",
        "DELAYED",
        "DELAY_KEY_WRITE",
        "DELETE",
        "DESC",
        "DESCRIBE",
        "DES_KEY_FILE",
        "DETERMINISTIC",
        "DIAGNOSTICS",
        "DIRECTORY",
        "DISABLE",
        "DISCARD",
        "DISK",
        "DISTINCT",
        "DISTINCTROW",
        "DIV",
        "DO",
        "DOUBLE",
        "DROP",
        "DUAL",
        "DUMPFILE",
        "DUPLICATE",
        "DYNAMIC",
        "EACH",
        "ELSE",
        "ELSEIF",
        "ENABLE",
        "ENCLOSED",
        "ENCRYPTION",
        "END",
        "ENDS",
        "ENGINE",
        "ENGINES",
        "ENUM",
        "ERROR",
        "ERRORS",
        "ESCAPE",
        "ESCAPED",
        "EVENT",
        "EVENTS",
        "EVERY",
        "EXCHANGE",
        "EXECUTE",
        "EXISTS",
        "EXIT",
        "EXPANSION",
        "EXPIRE",
        "EXPLAIN",
        "EXPORT",
        "EXTENDED",
        "EXTENT_SIZE",
        "FALSE",
        "FAST",
        "FAULTS",
        "FETCH",
        "FIELDS",
        "FILE",
        "FILE_BLOCK_SIZE",
        "FILTER",
        "FIRST",
        "FIXED",
        "FLOAT",
        "FLOAT4",
        "FLOAT8",
        "FLUSH",
        "FOLLOWS",
        "FOR",
        "FORCE",
        "FOREIGN",
        "FORMAT",
        "FOUND",
        "FROM",
        "FULL",
        "FULLTEXT",
        "FUNCTION",
        "GENERAL",
        "GENERATED",
        "GEOMETRY",
        "GEOMETRYCOLLECTION",
        "GET",
        "GET_FORMAT",
        "GLOBAL",
        "GRANT",
        "GRANTS",
        "GROUP",
        "GROUP_REPLICATION",
        "HANDLER",
        "HASH",
        "HAVING",
        "HELP",
        "HIGH_PRIORITY",
        "HOST",
        "HOSTS",
        "HOUR",
        "HOUR_MICROSECOND",
        "HOUR_MINUTE",
        "HOUR_SECOND",
        "IDENTIFIED",
        "IF",
        "IGNORE",
        "IGNORE_SERVER_IDS",
        "IMPORT",
        "IN",
        "INDEX",
        "INDEXES",
        "INFILE",
        "INITIAL_SIZE",
        "INNER",
        "INOUT",
        "INSENSITIVE",
        "INSERT",
        "INSERT_METHOD",
        "INSTALL",
        "INSTANCE",
        "INT",
        "INT1",
        "INT2",
        "INT3",
        "INT4",
        "INT8",
        "INTEGER",
        "INTERVAL",
        "INTO",
        "INVOKER",
        "IO",
        "IO_AFTER_GTIDS",
        "IO_BEFORE_GTIDS",
        "IO_THREAD",
        "IPC",
        "IS",
        "ISOLATION",
        "ISSUER",
        "ITERATE",
        "JOIN",
        "JSON",
        "KEY",
        "KEYS",
        "KEY_BLOCK_SIZE",
        "KILL",
        "LANGUAGE",
        "LAST",
        "LEADING",
        "LEAVE",
        "LEAVES",
        "LEFT",
        "LESS",
        "LEVEL",
        "LIKE",
        "LIMIT",
        "LINEAR",
        "LINES",
        "LINESTRING",
        "LIST",
        "LOAD",
        "LOCAL",
        "LOCALTIME",
        "LOCALTIMESTAMP",
        "LOCK",
        "LOCKS",
        "LOGFILE",
        "LOGS",
        "LONG",
        "LONGBLOB",
        "LONGTEXT",
        "LOOP",
        "LOW_PRIORITY",
        "MASTER",
        "MASTER_AUTO_POSITION",
        "MASTER_BIND",
        "MASTER_CONNECT_RETRY",
        "MASTER_DELAY",
        "MASTER_HEARTBEAT_PERIOD",
        "MASTER_HOST",
        "MASTER_LOG_FILE",
        "MASTER_LOG_POS",
        "MASTER_PASSWORD",
        "MASTER_PORT",
        "MASTER_RETRY_COUNT",
        "MASTER_SERVER_ID",
        "MASTER_SSL",
        "MASTER_SSL_CA",
        "MASTER_SSL_CAPATH",
        "MASTER_SSL_CERT",
        "MASTER_SSL_CIPHER",
        "MASTER_SSL_CRL",
        "MASTER_SSL_CRLPATH",
        "MASTER_SSL_KEY",
        "MASTER_SSL_VERIFY_SERVER_CERT",
        "MASTER_TLS_VERSION",
        "MASTER_USER",
        "MATCH",
        "MAXVALUE",
        "MAX_CONNECTIONS_PER_HOUR",
        "MAX_QUERIES_PER_HOUR",
        "MAX_ROWS",
        "MAX_SIZE",
        "MAX_STATEMENT_TIME",
        "MAX_UPDATES_PER_HOUR",
        "MAX_USER_CONNECTIONS",
        "MEDIUM",
        "MEDIUMBLOB",
        "MEDIUMINT",
        "MEDIUMTEXT",
        "MEMORY",
        "MERGE",
        "MESSAGE_TEXT",
        "MICROSECOND",
        "MIDDLEINT",
        "MIGRATE",
        "MINUTE",
        "MINUTE_MICROSECOND",
        "MINUTE_SECOND",
        "MIN_ROWS",
        "MOD",
        "MODE",
        "MODIFIES",
        "MODIFY",
        "MONTH",
        "MULTILINESTRING",
        "MULTIPOINT",
        "MULTIPOLYGON",
        "MUTEX",
        "MYSQL_ERRNO",
        "NAME",
        "NAMES",
        "NATIONAL",
        "NATURAL",
        "NCHAR",
        "NDB",
        "NDBCLUSTER",
        "NEVER",
        "NEW",
        "NEXT",
        "NO",
        "NODEGROUP",
        "NONBLOCKING",
        "NONE",
        "NOT",
        "NO_WAIT",
        "NO_WRITE_TO_BINLOG",
        "NULL",
        "NUMBER",
        "NUMERIC",
        "NVARCHAR",
        "OFFSET",
        "OLD_PASSWORD",
        "ON",
        "ONE",
        "ONLY",
        "OPEN",
        "OPTIMIZE",
        "OPTIMIZER_COSTS",
        "OPTION",
        "OPTIONALLY",
        "OPTIONS",
        "OR",
        "ORDER",
        "OUT",
        "OUTER",
        "OUTFILE",
        "OWNER",
        "PACK_KEYS",
        "PAGE",
        "PARSER",
        "PARSE_GCOL_EXPR",
        "PARTIAL",
        "PARTITION",
        "PARTITIONING",
        "PARTITIONS",
        "PASSWORD",
        "PHASE",
        "PLUGIN",
        "PLUGINS",
        "PLUGIN_DIR",
        "POINT",
        "POLYGON",
        "PORT",
        "PRECEDES",
        "PRECISION",
        "PREPARE",
        "PRESERVE",
        "PREV",
        "PRIMARY",
        "PRIVILEGES",
        "PROCEDURE",
        "PROCESSLIST",
        "PROFILE",
        "PROFILES",
        "PROXY",
        "PURGE",
        "QUARTER",
        "QUERY",
        "QUICK",
        "RANGE",
        "READ",
        "READS",
        "READ_ONLY",
        "READ_WRITE",
        "REAL",
        "REBUILD",
        "RECOVER",
        "REDOFILE",
        "REDO_BUFFER_SIZE",
        "REDUNDANT",
        "REFERENCES",
        "REGEXP",
        "RELAY",
        "RELAYLOG",
        "RELAY_LOG_FILE",
        "RELAY_LOG_POS",
        "RELAY_THREAD",
        "RELEASE",
        "RELOAD",
        "REMOVE",
        "RENAME",
        "REORGANIZE",
        "REPAIR",
        "REPEAT",
        "REPEATABLE",
        "REPLACE",
        "REPLICATE_DO_DB",
        "REPLICATE_DO_TABLE",
        "REPLICATE_IGNORE_DB",
        "REPLICATE_IGNORE_TABLE",
        "REPLICATE_REWRITE_DB",
        "REPLICATE_WILD_DO_TABLE",
        "REPLICATE_WILD_IGNORE_TABLE",
        "REPLICATION",
        "REQUIRE",
        "RESET",
        "RESIGNAL",
        "RESTORE",
        "RESTRICT",
        "RESUME",
        "RETURN",
        "RETURNED_SQLSTATE",
        "RETURNS",
        "REVERSE",
        "REVOKE",
        "RIGHT",
        "RLIKE",
        "ROLLBACK",
        "ROLLUP",
        "ROTATE",
        "ROUTINE",
        "ROW",
        "ROWS",
        "ROW_COUNT",
        "ROW_FORMAT",
        "RTREE",
        "SAVEPOINT",
        "SCHEDULE",
        "SCHEMA",
        "SCHEMAS",
        "SCHEMA_NAME",
        "SECOND",
        "SECOND_MICROSECOND",
        "SECURITY",
        "SELECT",
        "SENSITIVE",
        "SEPARATOR",
        "SERIAL",
        "SERIALIZABLE",
        "SERVER",
        "SESSION",
        "SET",
        "SHARE",
        "SHOW",
        "SHUTDOWN",
        "SIGNAL",
        "SIGNED",
        "SIMPLE",
        "SLAVE",
        "SLOW",
        "SMALLINT",
        "SNAPSHOT",
        "SOCKET",
        "SOME",
        "SONAME",
        "SOUNDS",
        "SOURCE",
        "SPATIAL",
        "SPECIFIC",
        "SQL",
        "SQLEXCEPTION",
        "SQLSTATE",
        "SQLWARNING",
        "SQL_AFTER_GTIDS",
        "SQL_AFTER_MTS_GAPS",
        "SQL_BEFORE_GTIDS",
        "SQL_BIG_RESULT",
        "SQL_BUFFER_RESULT",
        "SQL_CACHE",
        "SQL_CALC_FOUND_ROWS",
        "SQL_NO_CACHE",
        "SQL_SMALL_RESULT",
        "SQL_THREAD",
        "SQL_TSI_DAY",
        "SQL_TSI_HOUR",
        "SQL_TSI_MINUTE",
        "SQL_TSI_MONTH",
        "SQL_TSI_QUARTER",
        "SQL_TSI_SECOND",
        "SQL_TSI_WEEK",
        "SQL_TSI_YEAR",
        "SSL",
        "STACKED",
        "START",
        "STARTING",
        "STARTS",
        "STATS_AUTO_RECALC",
        "STATS_PERSISTENT",
        "STATS_SAMPLE_PAGES",
        "STATUS",
        "STOP",
        "STORAGE",
        "STORED",
        "STRAIGHT_JOIN",
        "STRING",
        "SUBCLASS_ORIGIN",
        "SUBJECT",
        "SUBPARTITION",
        "SUBPARTITIONS",
        "SUPER",
        "SUSPEND",
        "SWAPS",
        "SWITCHES",
        "TABLE",
        "TABLES",
        "TABLESPACE",
        "TABLE_CHECKSUM",
        "TABLE_NAME",
        "TEMPORARY",
        "TEMPTABLE",
        "TERMINATED",
        "TEXT",
        "THAN",
        "THEN",
        "TIME",
        "TIMESTAMP",
        "TIMESTAMPADD",
        "TIMESTAMPDIFF",
        "TINYBLOB",
        "TINYINT",
        "TINYTEXT",
        "TO",
        "TRAILING",
        "TRANSACTION",
        "TRIGGER",
        "TRIGGERS",
        "TRUE",
        "TRUNCATE",
        "TYPE",
        "TYPES",
        "UNCOMMITTED",
        "UNDEFINED",
        "UNDO",
        "UNDOFILE",
        "UNDO_BUFFER_SIZE",
        "UNICODE",
        "UNINSTALL",
        "UNION",
        "UNIQUE",
        "UNKNOWN",
        "UNLOCK",
        "UNSIGNED",
        "UNTIL",
        "UPDATE",
        "UPGRADE",
        "USAGE",
        "USE",
        "USER",
        "USER_RESOURCES",
        "USE_FRM",
        "USING",
        "UTC_DATE",
        "UTC_TIME",
        "UTC_TIMESTAMP",
        "VALIDATION",
        "VALUE",
        "VALUES",
        "VARBINARY",
        "VARCHAR",
        "VARCHARACTER",
        "VARIABLES",
        "VARYING",
        "VIEW",
        "VIRTUAL",
        "WAIT",
        "WARNINGS",
        "WEEK",
        "WEIGHT_STRING",
        "WHEN",
        "WHERE",
        "WHILE",
        "WITH",
        "WITHOUT",
        "WORK",
        "WRAPPER",
        "WRITE",
        "X509",
        "XA",
        "XID",
        "XML",
        "XOR",
        "YEAR",
        "YEAR_MONTH",
        "ZEROFILL",
    ]

    style_items = [
            (STC_MYSQL_DEFAULT,              'default'),
            (STC_MYSQL_COMMENT,              'comment'),
            (STC_MYSQL_COMMENTLINE,          'comment line'),
            (STC_MYSQL_DATABASEOBJECT,       'database object'),
            (STC_MYSQL_DQSTRING,             'double quoted string'),
            (STC_MYSQL_FUNCTION,             'function'),
            (STC_MYSQL_HIDDENCOMMAND,        'hidden command'),
            (STC_MYSQL_IDENTIFIER,           'identifier'),
            (STC_MYSQL_KEYWORD,              'keyword'),
            (STC_MYSQL_KNOWNSYSTEMVARIABLE,  'known system variable'),
            (STC_MYSQL_MAJORKEYWORD,         'major keyword'),
            (STC_MYSQL_NUMBER,               'number'),
            (STC_MYSQL_OPERATOR,             'operator'),
            (STC_MYSQL_PROCEDUREKEYWORD,     'procedure keyword'),
            (STC_MYSQL_QUOTEDIDENTIFIER,     'quoted identifier'),
            (STC_MYSQL_SQSTRING,             'single quoted string'),
            (STC_MYSQL_STRING,               'string'),
            (STC_MYSQL_SYSTEMVARIABLE,       'system variable'),
            (STC_MYSQL_USER1,                'user 1'),
            (STC_MYSQL_USER2,                'user 2'),
            (STC_MYSQL_USER3,                'user 3'),
            (STC_MYSQL_VARIABLE,             'mysql variable'),
    ]

    style = {
            'default': "fore:#FFFFFF, back:#000000",
            'comment': "fore:#7F7F7F,back:#FFFFFF,bold",
            'comment line': "fore:#7F7F7F,back:#FFFFFF,bold",
            'database object': "fore:#00007F,back:#FFFFFF,bold",
            'double quoted string': "fore:#7F0000,back:#FFFFFF,bold",
            'function': "fore:#00007F,back:#FFFFFF,bold",
            'hidden command': "fore:#00002F,back:#FFFFFF,bold",
            'identifier': "fore:#00007F,back:#FFFFFF,bold",
            'keyword': "fore:#00007F,back:#FFFFFF,bold",
            'known system variable': "fore:#00007F,back:#FFFFFF,bold",
            'major keyword': "fore:#00007F,back:#FFFFFF,bold",
            'number': "fore:#402200,back:#FFFFFF,bold",
            'operator': "fore:#224000,back:#FFFFFF,bold",
            'procedure keyword': "fore:#00007F,back:#FFFFFF,bold",
            'quoted identifier': "fore:#960000",
            'single quoted string': "fore:#960000",
            'string': "fore:#960000",
            'system variable': "fore:#00007F,back:#FFFFFF,bold",
            'user 1': "fore:#00007F,back:#FFFFFF,bold",
            'user 2': "fore:#00007F,back:#FFFFFF,bold",
            'user 3': "fore:#00007F,back:#FFFFFF,bold",
            'mysql variable': "fore:#00007F,back:#FFFFFF,bold",
        }

    def __init__(self, **kwargs):
        """Init"""
        kwargs['lexer'] = STC_LEX_MYSQL
        self._use_bookmarks = kwargs.get('use_bookmarks', False)
        self._use_breakpoints = kwargs.get('use_breakpoints', False)
        super(EditorHandler, self).__init__(**kwargs)

    def SetProperties(self, editor):
        """Set other properties"""
        editor.SetProperty("fold", "1")
        editor.SetProperty("fold.comment", "1")
        editor.SetProperty("fold.compact", "0")
        editor.SetProperty("fold.margin.width", "5")
        editor.SetProperty("indent.automatic", "1")
        editor.SetProperty("indent.opening", "1")
        editor.SetProperty("indent.closing", "1")
        editor.SetProperty("indent.size", "4")
        editor.SetProperty("tabsize", "4")
        editor.SetProperty("indent.size", "4")
        editor.SetProperty("use.tabs", "1")
        editor.SetProperty("tab.indents", "1")
        editor.SetIndent(4)
        editor.SetUseTabs(True)
        editor.SetTabIndents(True)
        editor.SetTabWidth(4)
        editor.SetWrapMode(False)
        editor.SetIndentationGuides(False)

    def SetupEvents(self, editor):
        """Set event handling"""
        editor.CmdKeyAssign(ord('+'), STC_SCMOD_CTRL, STC_CMD_ZOOMIN)
        editor.CmdKeyAssign(ord('-'), STC_SCMOD_CTRL, STC_CMD_ZOOMOUT)
        editor.Bind(wx.EVT_MENU, editor.ToggleBookmark, id=BOOKMARK_TOGGLE)
        editor.Bind(wx.EVT_MENU, editor.PrevBookmark, id=BOOKMARK_UP)
        editor.Bind(wx.EVT_MENU, editor.NextBookmark, id=BOOKMARK_DOWN)
        editor.Bind(wx.EVT_MENU, editor.Find, id=FIND)
        editor.Bind(wx.EVT_MENU, editor.FindNext, id=FIND_NEXT)
        editor.Bind(wx.EVT_MENU, editor.FindPrevious, id=FIND_PREV)
        editor.Bind(EVT_STC_MARGINCLICK, editor.OnMarginClick)
        editor.Bind(wx.EVT_KEY_UP, editor.OnKey)
        editor.Bind(EVT_STC_UPDATEUI, editor.OnUpdateEditUI)

        aTable = wx.AcceleratorTable([
            #wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_F9, BREAKPOINT_TOGGLE), <- mus be handled by the view
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_F2, BOOKMARK_TOGGLE),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, wx.WXK_UP, BOOKMARK_UP),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, wx.WXK_DOWN, BOOKMARK_DOWN),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('F'), FIND),
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_F3, FIND_NEXT),
            wx.AcceleratorEntry(wx.ACCEL_SHIFT, wx.WXK_F3, FIND_PREV),
        ])
        editor.SetAcceleratorTable(aTable)

    def Initialize(self, editor):
        """Base initialization"""
        editor.StyleClearAll()
        editor.SetLexer(self.lexer)
        editor.SetCodePage(STC_CP_UTF8)
        editor.EnableBreakpoints(self._use_breakpoints)
        editor.EnableBookmarks(self._use_bookmarks)
        editor.SetStyleBits(editor.GetStyleBitsNeeded())
        self.init_fonts(editor)
        editor.SetExtraAscent(6)
        super(EditorHandler, self).Initialize(editor)



"""Subclass of PyMethodPane, which is generated by wxFormBuilder."""

import wx.aui
import wx.stc
from ctx import theContext as context
import tran
from app.ui import pane
from app.handlers import IdentifiersHandler as ID
from activity.models.handlers.py import EditorHandler
from activity.models.ui import ui as ui


# Implementing PyMethodPane
class PyMethodPane(ui.PyMethodPane):
    """Implements method pane editor"""

    #command ids: run
    _closePane  = ID.register('close-pane')
    _saveCode = ID.new()
    _undoCode = ID.register('undo')
    _redoCode = ID.register('redo')
    _copyCode = ID.register('copy')
    _cutCode = ID.register('cut')
    _pasteCode = ID.register('paste')
    _deleteCode = ID.register('delete')
    _toggleFull = ID.register('fullscreen')
    _navigator = ID.new()

    def _create_toolbars(self):
        """Create custom toolbar"""
        self.m_localToolbar = wx.aui.AuiToolBar(self, wx.ID_ANY, wx.DefaultPosition,
            wx.DefaultSize, wx.aui.AUI_TB_HORZ_LAYOUT)
        self.m_save = self.m_localToolbar.AddTool(self._saveCode,
            u"save code", wx.ArtProvider.GetBitmap(wx.ART_FILE_SAVE, wx.ART_TOOLBAR),
            wx.NullBitmap, wx.ITEM_NORMAL, u"Save", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_undo = self.m_localToolbar.AddTool(self._undoCode, u"tool",
            wx.ArtProvider.GetBitmap(wx.ART_UNDO, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Undo", wx.EmptyString, None)
        self.m_redo = self.m_localToolbar.AddTool(self._redoCode, u"tool",
            wx.ArtProvider.GetBitmap(wx.ART_REDO, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Redo", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_copy = self.m_localToolbar.AddTool(self._copyCode, u"copy",
            wx.ArtProvider.GetBitmap(wx.ART_COPY, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Copy", wx.EmptyString, None)
        self.m_cut = self.m_localToolbar.AddTool(self._cutCode, u"cut", wx.ArtProvider.GetBitmap(
            wx.ART_CUT, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL, u"Cut", wx.EmptyString, None)
        self.m_paste = self.m_localToolbar.AddTool(self._pasteCode, u"paste",
            wx.ArtProvider.GetBitmap(wx.ART_PASTE, wx.ART_TOOLBAR), wx.NullBitmap,
            wx.ITEM_NORMAL, u"Paste", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_delete = self.m_localToolbar.AddTool(self._deleteCode, u"delete",
            wx.ArtProvider.GetBitmap(wx.ART_DELETE, wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Delete", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_fullScreen = self.m_localToolbar.AddTool(self._toggleFull, u"tool",
            wx.ArtProvider.GetBitmap(u"gtk-fullscreen", wx.ART_TOOLBAR), wx.NullBitmap, wx.ITEM_NORMAL,
            u"Toggle fullscreen", wx.EmptyString, None)
        self.m_localToolbar.AddSeparator()
        self.m_navigator = self.m_localToolbar.AddTool(self._navigator, u"tool", wx.Bitmap(u"app/res/method.xpm",
            wx.BITMAP_TYPE_ANY), wx.NullBitmap, wx.ITEM_NORMAL, u"insert call", wx.EmptyString, None)
        self.m_localToolbar.Realize()
        self.m_toolbarSizer.Add(self.m_localToolbar, 0, wx.ALL, 5)
        self.Layout()
        # Connect Events
        self.m_editor.Bind(wx.EVT_KILL_FOCUS, self.OnLeaveFocus)
        self.m_editor.Bind(wx.EVT_SET_FOCUS, self.OnGetFocus)

    def __init__(self, parent, mainframe, method):
        """Intialization of method editor"""
        self._editorArgs = {
            'language': 'python',
            'handler': EditorHandler(text=method._content,
                use_bookmarks=False,
                use_breakpoints=False,
                read_only=method._readOnly)
        }
        super(PyMethodPane, self).__init__(parent)
        self._mainframe = mainframe
        self._object = method
        self._types = {}
        self._notebook = parent

        """
        if method.note:
            self.m_editor.AnnotationSetVisible(wx.stc.STC_ANNOTATION_BOXED)
            self.m_editor.AnnotationSetText(0, method._note)"""
        #self.m_editor.SendMsg(2540, wp="Prueba de anotacion", lp=long(0))
        #self.m_editor.SendMsg(2548, wp=1, lp=long(0))
        self.Layout()
        self._create_toolbars()
        wx.stc.EVT_STC_CHANGE(self.m_editor,
            self.m_editor.GetId(), self.OnEditorChange)

        #self.Bind(EVT_STC_STYLENEEDED, self.handleStyleNeeded,
        #id=self.m_editor.GetId())
        frame = context.frame

        self.Bind(wx.EVT_MENU, frame.CloseCurrentDocPane, id=self._closePane)
        self.Bind(wx.EVT_MENU, self.OnDeleteCode, id=self._deleteCode)
        self.Bind(wx.EVT_MENU, self.OnUndoCode, id=self._undoCode)
        self.Bind(wx.EVT_MENU, self.OnRedoCode, id=self._redoCode)
        self.Bind(wx.EVT_MENU, self.OnCopyCode, id=self._copyCode)
        self.Bind(wx.EVT_MENU, self.OnPasteCode, id=self._pasteCode)
        self.Bind(wx.EVT_MENU, self.OnCutCode, id=self._cutCode)
        self.Bind(wx.EVT_MENU, self.OnSaveCode, id=self._saveCode)
        self.Bind(wx.EVT_MENU, self.OnFullScreen, id=self._toggleFull)

        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateDeleteCode, id=self._deleteCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUndoCode, id=self._undoCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateRedoCode, id=self._redoCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateCopyCode, id=self._copyCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdatePasteCode, id=self._pasteCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateCutCode, id=self._cutCode)
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateSaveCode, id=self._saveCode)

        self.m_editor.popup_handler = self
        self._fullScreen = None

        aTable = wx.AcceleratorTable([
            wx.AcceleratorEntry(wx.ACCEL_ALT | wx.ACCEL_CTRL, ord('F'), self._toggleFull),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('W'), self._closePane),
            wx.AcceleratorEntry(wx.ACCEL_CTRL, ord('S'), self._saveCode),
            wx.AcceleratorEntry(wx.ACCEL_NORMAL, wx.WXK_DELETE, self._deleteCode)
        ])
        self.SetAcceleratorTable(aTable)

    def Refresh(self):
        """Update editor from external changes (like undo/redo)"""
        line = self.m_editor.GetFirstVisibleLine()
        pos = self.m_editor.GetCurrentPos()
        self.m_editor.Freeze()
        self.m_editor.SetText(self._object._content)
        self.m_editor.SetModified(False)
        try:
            self.m_editor.SetFirstVisibleLine(line)
            self.m_editor.SetCurrentPos(pos)
        except:
            pass
        self.m_editor.Thaw()

    def PreDelete(self):
        """Remove toolbar first in order to avoid gtk-collision at close"""
        #self.m_editor.Destroy()
        self.m_localToolbar.Destroy()

    def Popup(self, editor, text=None):
        """Shows a custom popup"""
        if text:
            # in the future, we need to show a complete floating dialog here
            # we receive "this->" or "object." here ..
            return
        menu = wx.Menu()
        menu.Append(self._mainframe.editProperties.GetId(),
            "test menu", "Edit relation properties")
        pos = editor.GetCurrentPos()
        p = editor.PointFromPosition(pos)
        editor.PopupMenu(menu, p)
        menu.Destroy()

    def OnNavigator(self, event):
        """Navigate throug the class structure"""
        pos = self.m_editor.GetCurrentPos()
        p = self.m_editor.PointFromPosition(pos)
        p = self.m_editor.ClientToScreen(p)
        dialog = app.ui.dlg.CodeNavigator(self, self._object)
        dialog.Move(p)
        if dialog.ShowModal() != wx.ID_OK:
            return
        pass

    def OnUndoCode(self, event):
        """Edit undo"""
        self.m_editor.Undo()

    def OnRedoCode(self, event):
        """Edit redo"""
        self.m_editor.Redo()

    def OnUpdatePasteCode(self, event):
        """Edit paste?"""
        event.Enable(self.m_editor.CanPaste())

    def OnUpdateCopyCode(self, event):
        """Edit copy?"""
        (b, e) = self.m_editor.GetSelection()
        event.Enable(b != e)

    def OnUpdateCutCode(self, event):
        """Edit cut?"""
        if self._object._readOnly:
            event.Enable(False)
        else:
            (b, e) = self.m_editor.GetSelection()
            event.Enable(b != e)

    def OnUpdateRedoCode(self, event):
        """Edit redo?"""
        if self.m_editor.CanRedo():
            event.SetText("Redo edit")
            event.Enable(True)
        else:
            event.SetText("Can't redo")
            event.Enable(False)

    def OnUpdateUndoCode(self, event):
        """Edit undo?"""
        if self.m_editor.CanUndo():
            event.SetText("Undo edit")
            event.Enable(True)
        else:
            event.SetText("Can't undo")
            event.Enable(False)

    def OnUpdateDeleteCode(self, event):
        """Edit delete?"""
        event.Enable(not self._object._readOnly)

    def Enabled(self, event):
        """Joker event"""
        event.Enable(True)

    def Disabled(self, event):
        """Joker event"""
        event.Enable(False)

    def UpdateModifiedTitle(self):
        """Update the tile after modifications"""
        if not self._fullScreen:
            i = self._notebook.GetPageIndex(self)
            s = self._notebook.GetPageText(i)
            m = '[modified] '
            if self.m_editor.IsModified():
                if s.find(m) == wx.NOT_FOUND:
                    self._notebook.SetPageText(i, m + s)
            else:
                if s.find(m) != wx.NOT_FOUND:
                    s = s.replace(m, '')
                    self._notebook.SetPageText(i, s)

    def OnEditorChange(self, event):
        """Called when editor status change"""
        if not self._fullScreen:
            self.UpdateModifiedTitle()

    @tran.TransactionalMethod('edit method')
    def Commit(self):
        """Update document"""
        text = self.m_editor.GetText()
        o = self._object
        if text == o._content:
            return False
        o.SaveState()
        o._content = text
        o.project.SetModified(True)
        k = o.inner_module or o.inner_package
        if k:
            k.ExportPythonCodeFiles()
        return True

    def OnDeleteCode(self, event):
        """Handle delete key"""
        wx.Bell()
        (b, e) = self.m_editor.GetSelection()
        if b == e:
            self.m_editor.CharRight()
        self.m_editor.DeleteBack()

    def OnCopyCode(self, event):
        """Handle a copy event"""
        self.m_editor.Copy()

    def OnPasteCode(self, event):
        """Handle a paste event"""
        self.m_editor.Paste()

    def OnCutCode(self, event):
        """Handle a paste event"""
        self.m_editor.Cut()

    def OnUpdateSaveCode(self, event):
        """Handles update event"""
        event.Enable(self.m_editor.IsModified())

    def OnSaveCode(self, event):
        """Handles save event"""
        self.Commit()
        self.m_editor.ResetModified()
        self.UpdateModifiedTitle()

    def OnFullScreen(self, event):
        """ShowFullscreen"""
        if self._fullScreen:
            self._fullScreen.LeaveFullScreen()
            wx.YieldIfNeeded()
            #self._fullScreen.Destroy()
            self._fullScreen.Close()
            self._fullScreen = None
            self.UpdateModifiedTitle()
        else:
            self._fullScreen = pane.FullScreen(self)
        self.m_editor.SetFocus()

    def handleStyleNeeded(self, event):
        """Handle scintylla events"""
        wnd = self.m_editor
        pos = wnd.GetCurrentPos()
        if pos == 0:
            return
        if wnd.GetCharAt(pos - 1) == wx.T(':'):
            lnno = wnd.GetCurrentLine()
            iden = wnd.GetLineIndentation(lnno - 1)
            wnd.SetLineIndentation(lnno, iden)

            # = self.m_editor.GetFoldLevel(lnno - 1)
        # if this is not a normal line
        #self.m_editor.SetFoldLevel(lnno,LastIndent)
        #pass


    #def AutoIdent(self, event):
    #    """Handle enter"""
    #    key = event.GetKeyCode()
    #    wnd = self.m_editor
    #    if key == wx.WXK_NUMPAD_ENTER or key == wx.WXK_RETURN:
    #        if wnd.GetCharAt(wnd.GetCurrentPos() - 1) == 58:
    #            line = wnd.GetCurrentLine() + 1
    #            wnd.SetLineIndentation(line,
    #                wnd.GetLineIndentation(line) + wnd.GetTabWidth())
    #    event.Skip()






"""Subclass of WebPreferences, which is generated by wxFormBuilder."""

import wx
import app

# Implementing WebPreferences
class WebPreferences( app.WebPreferences ):
	def __init__( self, parent, **kwargs ):
		self._use_proxy = kwargs.get('use_proxy',False)
		self._http_proxy = kwargs.get('http_proxy','')
		self._https_proxy = kwargs.get('https_proxy','')
		self._ftp_proxy = kwargs.get('ftp_proxy','')
		self._use_auto = kwargs.get('auto',True)
		# -- enable this when multiproxy is done:
		# self._use_same = kwargs.get('same',False)
		# -- delete this when multiproxy is done:
		self._use_same = True
		super(WebPreferences, self).__init__(parent)
		self.UpdateData()
	
	def UpdateData(self):
		"""Setup control values"""
		if self._use_proxy:
			self.m_radioBtn5.SetValue(True)
			self.st_http_proxy.Enable(True)
			self.m_http_proxy.Enable(True)
			# -- enable this when multiproxy is done
			# self.m_checkBox6.Enable(True)
			# detailed = not self.m_checkBox6.GetValue()
			# self.st_https_proxy.Enable(detailed)
			# self.m_https_proxy.Enable(detailed)
			# self.st_ftp_proxy.Enable(detailed)
			# self.m_ftp_proxy.Enable(detailed)
		else:
			self.m_radioBtn4.SetValue(True)
			self.st_http_proxy.Enable(False)
			self.m_http_proxy.Enable(False)
			self.m_checkBox6.Enable(False)
			self.st_https_proxy.Enable(False)
			self.m_https_proxy.Enable(False)
			self.st_ftp_proxy.Enable(False)
			self.m_ftp_proxy.Enable(False)
		self.m_http_proxy.SetValue(self._http_proxy)
		self.m_https_proxy.SetValue(self._https_proxy)
		self.m_ftp_proxy.SetValue(self._ftp_proxy)
		self.m_checkBox6.SetValue(self._use_same)
		
	def GetData(self):
		"""Get data from controls"""
		self._use_proxy = self.m_radioBtn5.GetValue()
		self._http_proxy = self.m_http_proxy.GetValue()
		self._https_proxy = self.m_https_proxy.GetValue()
		self._ftp_proxy = self.m_ftp_proxy.GetValue()
		# -- enable this when multiproxy is done
		# self._use_same = self.m_checkBox6.GetValue()		
		
	# Handlers for WebPreferences events.
	def OnAutoNetwork( self, event ):
		# disable all controls
		self.st_http_proxy.Enable(False)
		self.m_http_proxy.Enable(False)
		# -- enable this when multiproxy is done
		# self.m_checkBox6.Enable(False)
		# self.st_https_proxy.Enable(False)
		# self.m_https_proxy.Enable(False)
		# self.st_ftp_proxy.Enable(False)
		# self.m_ftp_proxy.Enable(False)
		self.m_button4.Enable(True)
	
	def OnManualProxy( self, event ):
		self.st_http_proxy.Enable(True)
		self.m_http_proxy.Enable(True)
		# -- enable this when multiproxy is done
		# self.m_checkBox6.Enable(True)
		# detailed = not self.m_checkBox6.GetValue()
		# self.st_https_proxy.Enable(detailed)
		# self.m_https_proxy.Enable(detailed)
		# self.st_ftp_proxy.Enable(detailed)
		# self.m_ftp_proxy.Enable(detailed)
		self.m_button4.Enable(True)
	
	def OnChangeHttpProxy( self, event ):
		self.m_button4.Enable(True)
	
	def OnSameProxy( self, event ):
		# -- enable this when multiproxy is done
		# detailed = not self.m_checkBox6.GetValue()
		# self.st_https_proxy.Enable(detailed)
		# self.m_https_proxy.Enable(detailed)
		# self.st_ftp_proxy.Enable(detailed)
		# self.m_ftp_proxy.Enable(detailed)
		# self.m_button4.Enable(True)
		pass
	
	def OnChangeHttpsProxy( self, event ):
		self.m_button4.Enable(True)
	
	def OnChangeFtpProxy( self, event ):
		self.m_button4.Enable(True)

	def OnApply( self, event ):
		import ctypes
		import platform
		self.GetData()
		s_plat = platform.platform() 
		if 'Linux' not in s_plat and 'linux' not in s_plat:
			wx.MessageBox('Proxy is only supported in linux','Error',
				wx.OK | wx.CENTER | wx.ICON_ERROR, self)
			return
		if self._use_proxy:
			try:	
				soup = ctypes.cdll.LoadLibrary('libsoup-2.4.so')
				webkit = ctypes.cdll.LoadLibrary('libwebkitgtk-1.0.so')
				gobject = ctypes.cdll.LoadLibrary('libgobject-2.0.so')
				if '64bit' in platform.architecture():
					webkit.webkit_get_default_session.restype = ctypes.c_uint64
					soup.soup_uri_new.restype = ctypes.c_uint64
					gobject.g_object_set.argtypes=(ctypes.c_uint64, ctypes.c_char_p, ctypes.c_uint64, ctypes.c_uint64)
					soup.soup_uri_free.argtypes=(ctypes.c_uint64,)
				else:
					#untested
					webkit.webkit_get_default_session.restype = ctypes.c_uint32
					soup.soup_uri_new.restype = ctypes.c_uint32
					gobject.g_object_set.argtypes=(ctypes.c_uint32, ctypes.c_char_p, ctypes.c_uint32, ctypes.c_uint32)
					soup.soup_uri_free.argtypes=(ctypes.c_uint32,)
				proxy_uri = soup.soup_uri_new('http://{proxy}'.format(proxy=self._http_proxy))
				session = webkit.webkit_get_default_session()
				gobject.g_object_set(session, "proxy-uri", proxy_uri, 0)
				soup.soup_uri_free(proxy_uri)
			except:
				wx.MessageBox('Failed','Error', wx.OK | wx.CENTER | wx.ICON_ERROR, self)
				return
		self.m_button4.Enable(False)

	



"""Subclass of DatabasePane, which is generated by wxFormBuilder."""
import wx
import wx.stc
from model import database
from app.handlers import IdentifiersHandler as ID
from activity.models.ui import ui as ui
from activity.models.handlers.sql import EditorHandler
from  ctx import theContext as context



# Implementing DatabasePane
class DatabasePane(ui.DatabasePane):
    """Implements database sql editor"""
    _closePane  = ID.register('close-pane')

    def __init__(self, parent, mainframe, object):
        """Initialize pane"""
        if object and type(object) in [database.Table, database.Field, database.Schema]:
            text = object.query
        else:
            text = ''
        self._editorArgs = {
            'language': 'sql',
            'handler': EditorHandler(text=text)
        }
        super(DatabasePane, self).__init__(parent)
        self._mainframe = mainframe
        self._object = object
        self._notebook = parent
        self._handler = None
        if object and type(object) in [database.Table, database.Field, database.Schema]:
            self._handler = object.project.handler
            try:
                self.ExecuteSQL()
            except:
                pass
        self.m_editor.Bind(wx.EVT_KILL_FOCUS, self.OnLeaveFocus)
        self.m_editor.Bind(wx.EVT_SET_FOCUS, self.OnGetFocus)

    def ExecuteSQL(self):
        """Try to execute sql"""
        try:
            dc = wx.ClientDC(self.m_grid)
            font = self.m_grid.GetFont()
            with database.dbopen(self._handler) as conn:
                with database.cursorquery(conn, self.m_editor.GetText()) as result:
                    # Ok, get the columns
                    self.m_grid.ClearColumns()
                    self.m_grid.DeleteAllItems()
                    for x in result.description:
                    #w, h = dc.GetTextExtent(x[0])
                        w = dc.GetFullTextExtent(x[0], font)[0]
                        w = max(w + 40, 40)
                        self.m_grid.AppendTextColumn(x[0], 0, w, wx.ALIGN_LEFT, 7)
                        # Iterate results
                    row = result.fetchone()
                    while row is not None:
                        #row = [str(x) for x in row[0]]
                        self.m_grid.AppendItem(row)
                        row = result.fetchone()
                """
                with database.rawquery(conn, self.m_editor.GetText()) as data:
                    # Ok, get the columns
                    self.m_grid.ClearColumns()
                    self.m_grid.DeleteAllItems()
                    for x in data.describe():
                        #w, h = dc.GetTextExtent(x[0])
                        w = dc.GetFullTextExtent(x[0], font)[0]
                        w = max(w + 40, 40)
                        self.m_grid.AppendTextColumn(x[0], 0, w, wx.ALIGN_LEFT, 7)
                    # Iterate results
                    for i in range(0, data.num_rows()):
                        row = [str(x) for x in data.fetch_row()[0]]
                        self.m_grid.AppendItem(row)"""
            self.m_grid.Layout()
        except:
            pass

    def OnLeaveFocus(self, event):
        """"""
        self.m_editor.SetSTCFocus(False)
        pass

    def OnGetFocus(self, event):
        """"""
        self.m_editor.SetSTCFocus(True)
        #self.m_editor.SetFocus()
        pass

    def Commit(self):
        pass



# -*- coding: utf-8 -*-

import os
import xml.etree.ElementTree as et
from posix import R_OK

class xmlmetadoc(object):
    """This class represents a xml document containing definitions"""
    def __init__(self, partname, **kwargs):
        """constructs the object from the document part name and version"""
        self.ok = False
        path = os.path.join(os.getcwd(), 'plugin','models','relation',
            kwargs.get('version', 'standard'), partname)
        if not os.path.isfile(path):
            print('file {0} is missing'.format(path))
            return 
        if not os.access(path, R_OK):
            print('file {0} is not readable'.format(path))
            return 
        try:
            self.dictionary = kwargs.get('dictionary', {})
            self.target = self.dictionary.get('parent', None)
            if self.target is None:
                print('missing parent target reading definitions from {0}'.format(path))
                return 
            self.tree = et.parse(path)  # retorna et.ElementTree
            self.root = self.tree.getroot()  # retorna et.Element
            parent = self.dictionary.get('parent', None)
            self.filter = kwargs.get('filter', None)
            self.ok = True
            self.load_definitions()
        except:
            self.ok = False
            print('failed parsing xml file {0}'.format(path))
            
    def filter_set(self, component_set):
        """Evaluate a set, filter out the invalid components, 
        and return a list of (expression, puntuation) for
        the remaining components"""
        result = []
        for component in component_set:
            puntuation = 0
            filter = component.get('filter', None)
            if filter is None:
                result.append((component, 0))
                continue
            filter = filter.strip()[1:-1]  #  { and } left
            filtered_out = False
            for condition in filter.split(','):
                clause, value = condition.split(':')
                if value in ['True', 'False']:
                    value = eval(value)
                if clause in self.filter:
                    if self.filter[clause] != value:
                        filtered_out = True
                        break
                    puntuation += 1
            if not filtered_out:
                result.append((component, puntuation))
        # sort with better first
        result.sort(key=lambda x:-x[1])
        return result
    
    def better(self, component_set):
        """Apply a filter_set and select any best component. 
        Return None if no option is found"""
        evaluated = self.filter_set(component_set)
        if len(evaluated) == 0:
            return None
        return  evaluated[0][0]
            
    def load_definitions(self):
        """Read the definitions inside the document"""
        if self.ok is False:
            return
        definition_set = self.root.find('definitions')
        if definition_set is None:
            return
        for definition in definition_set:
            text = definition.findtext('.')
            for item, expression in [text.split('=')]:
                value = expression.format(**self.dictionary)
                if value in ['True', 'False']:
                    value = eval(value)
                self.dictionary[item] = value
                    
    def load_types(self, container = None, container_instance = None):
        import model
        if container is None:
            container = self.root
        type_set = container.find('types') or []
        if type_set is None:
            return        
        if container_instance is None:
            container_instance = self.target
        for type_ in type_set:
            kwargs = {}
            for arg in type_.keys():
                value = type_.get(arg).format(**self.dictionary)
                if value in ['True', 'False']:
                    value = eval(value)
                kwargs[arg] = value
            kwargs.update({'parent': container_instance, 'readonly': True})
            type_instance  = model.cc.Type(**kwargs)
            self.dictionary.update({type_instance.name: type_instance})
                        
    def load_members(self, container=None, container_instance=None):
        """Read the members from the document"""
        import model
        if container is None:
            container = self.root
        if container_instance is None:
            container_instance = self.target 
        member_set = container.find('members')
        if member_set is None:
            return
        for member in member_set:
            kwargs = {}
            for arg in member.keys():
                expression = member.get(arg).format(**self.dictionary)
                if arg == 'type' or expression in ['True', 'False']:
                    kwargs[arg] = eval(expression, self.dictionary)
                else:
                    kwargs[arg] = expression
            if 'type' in kwargs:
                kwargs['type'] = model.cc.typeinst(**kwargs)
            kwargs.update({'parent': container_instance, 'readonly': True})
            element = model.cc.MemberData(**kwargs)
            #  some elements needs further named references from runtime container
            if 'handler' in kwargs:
                setattr(container_instance, kwargs['handler'], element)
                
    def load_arguments(self, container, container_instance):
        """Read the arguments from the document"""
        import model
        argument_set = container.find('args')
        if argument_set is None:
            return 
        for argument in argument_set:
            kwargs = {}
            for arg in argument.keys():
                value = argument.get(arg).format(**self.dictionary)
                if arg == 'type' or value in ['True', 'False']:
                    kwargs[arg] = eval(value, self.dictionary)
                else:
                    kwargs[arg] = value
            kwargs['type'] = model.cc.typeinst(**kwargs)
            kwargs.update({'parent': container_instance, 'readonly': True})
            model.cc.Argument(**kwargs)        
                
    def load_constructors(self, container = None, container_instance = None):
        """Read the elements from the document"""
        import model
        if container is None:
            container = self.root
        constructor_set = container.find('ctors') or container.find('constructors')
        if constructor_set is None:
            return
        if container_instance is None:
            container_instance = self.target
        for constructor in constructor_set:
            kwargs = {}
            for arg in constructor.keys():
                expression = constructor.get(arg).format(**self.dictionary)
                if expression in ['True', 'False']:
                    kwargs[arg] = eval(expression, self.dictionary)
                else:
                    kwargs[arg] = expression
            init_set = constructor.findall('init')
            if init_set is not None:
                init = self.better(init_set)
                if init is not None:
                    kwargs['init'] = init.findtext('.').format(**self.dictionary)
            contents_set = constructor.findall('content')
            if contents_set is None:
                kwargs['content'] = ''
            else:
                content = self.better(contents_set)
                if content is not None:
                    kwargs['content'] = content.findtext('.').format(**self.dictionary)
                else:
                    kwargs['content'] = ''
            kwargs.update({'parent': container_instance, 'readonly': True})
            constructor_instance = model.cc.Constructor(**kwargs)
            # tomamos los argumentos
            self.load_arguments(constructor, constructor_instance)
            
    def load_methods(self, container = None, container_instance = None):
        """Read the elements from the document"""
        import model
        if container is None:
            container = self.root
        method_set = container.find('methods')
        if method_set is None:
            return
        if container_instance is None:
            container_instance = self.target
        for method in method_set:
            kwargs = {}
            for arg in method.keys():
                expression = method.get(arg).format(**self.dictionary)
                if arg == 'type':
                    if len(expression):
                        value = eval(expression, self.dictionary)
                    else:
                        value = self.dictionary[expression]  # implicit type
                elif expression in ['True', 'False']:
                    value = eval(expression, self.dictionary)
                else:
                    value = expression
                kwargs[arg] = value
                    
            contents_set = method.findall('content')
            if contents_set is None:
                kwargs['content'] = ''
            else:
                content = self.better(contents_set)
                if content is not None:
                    kwargs['content'] = content.findtext('.').format(**self.dictionary)
                else:
                    kwargs['content'] = ''
            kwargs['type'] = model.cc.typeinst(**kwargs)
            kwargs.update({'parent': container_instance, 'readonly': True})
            method_instance = model.cc.MemberMethod(**kwargs)
            # tomamos los argumentos
            self.load_arguments(method, method_instance)
            
    def load_destructor(self, container = None, container_instance = None):
        """Read the elements from the document"""
        import model
        if container is None:
            container = self.root
        destructor = container.find('dtor') or container.find('destructor') 
        if destructor is None:
            return
        if container_instance is None:
            container_instance = self.target
        kwargs = {}
        for arg in destructor.keys():
            expression = destructor.get(arg).format(**self.dictionary)
            if expression in ['True', 'False']:
                kwargs[arg] = eval(expression, self.dictionary)
            else:
                kwargs[arg] = expression                    
            contents_set = destructor.findall('content')
            if contents_set is None:
                kwargs['content'] = ''
            else:
                content = self.better(contents_set)
                if content is not None:
                    kwargs['content'] = content.findtext('.').format(**self.dictionary)
                else:
                    kwargs['content'] = ''
            kwargs.update({'parent': container_instance, 'readonly': True})
            method_instance = model.cc.Destructor(**kwargs)
            
    def load_classes(self):
        """Read the classes from the document"""
        import model
        classes_set = self.root.find('classes')
        if classes_set is None:
            return
        for class_ in classes_set:
            kwargs = {}
            # read class attributes
            for arg in class_.keys():
                value = class_.get(arg).format(**self.dictionary)
                if value in ['True', 'False']:
                    value = eval(value)
                kwargs[arg] = value
            # set aguments
            kwargs.update({'parent': self.target, 'readonly': True})
            # setup
            class_instance = model.cc.Class(**kwargs)
            self.dictionary.update({class_instance.name: class_instance})
            # find types
            self.load_types(class_, class_instance)
            # find los miembros
            self.load_members(class_, class_instance)
            # ctors
            self.load_constructors(class_, class_instance)
            # methods
            self.load_methods(class_, class_instance)
            #destructor
            self.load_destructor(class_, class_instance)
            

def relationMembers(**kwargs):
    """
    Este metodo lee la definicion de los metodos de un
    lado de la relacion y los crea

    Parametros:
        version: version de metodos por defecto, standard
        part: lado de la relacion (to o from)
        filter: dimodelionario de booleanos de selemodelion de version
        dictionary: dimodelionario de valores a utilizar en la evaluacion

    """
    doc = '{0}Members.xml'.format(kwargs['part'])
    meta = xmlmetadoc(doc,**kwargs)
    if meta.ok:
        meta.load_members()


def relationMethods(**kwargs):
    """
    Este metodo lee la definicion de los metodos de un
    lado de la relacion y los crea.

    Parametros:
        version: version de metodos por defecto, standard
        part: lado de la relacion (to o from)
        filter: dimodelionario de booleanos de selemodelion de version
        dictionary: dimodelionario de valores a utilizar en la evaluacion

    Retorna:

        None en caso de error
        Array de definicion de metodos.

    Comentarios:

        Los parametros de filtro permiten seleccionar entre
        diferentes versiones por ejemplo, de compatibilidad C++
        o niveles de seguridad (criticos), implementacion estatica
        presencia de metodos de busqueda, etc. Se selemodeliona
        el mejor matching y se excluyen aquellos metodos que
        no tengan ninguna version compatible con los filtros.

        Respecto de los simbolos utilizados en las declaraciones,
        esto es un aspecto pendiente de normalizar y documentar,
        ya que por ahora algunos de ellos dependen de la implementacion
        de los valores miembro.


    """
    import model
    doc = '{0}Methods.xml'.format(kwargs['part'])
    meta = xmlmetadoc(doc,**kwargs)
    if meta.ok:
        meta.load_methods()


def relationClasses(**kwargs):
    """
    Realiza la construccion de las clases anidadas
    definidas por la relacion
    """
    import model
    doc = '{0}Classes.xml'.format(kwargs['part'])
    meta = xmlmetadoc(doc,**kwargs)
    if meta.ok:
        meta.load_classes()
        
#     path = os.path.join(os.getcwd(), 'plugin', 'models', 'relation',
#         kwargs.get('version', 'standard'), doc)
#     key = kwargs.get('filter', None)
#     if not os.path.isfile(path):
#         print('{0} is missing'.format(path))
#         return
#     try:
#         tree = et.parse(path)
#         root = tree.getroot()
#         dictionary = kwargs.get('dictionary', {})
#         parent = dictionary.get('parent', None)
#         if not parent:
#             print('parent is missing')
#             return
#         # read definitions and add it to dict
#         defs = root.find('definitions')
#         if defs is not None:
#             for d in defs:
#                 text = d.findtext('.')
#                 for x, y in [text.split('=')]:
#                     v = y.format(**dictionary)
#                     if v in ['True', 'False']:
#                         v = eval(v)
#                     dictionary[x] = v
# 
#         classes = root.find('classes')
#         if classes is None:
#             print('missing classes')
#             return
#         for _class in classes:
#             keys = {}
#             # leemos los atributos de la clase
#             for x in _class.keys():
#                 keys[x] = _class.get(x).format(**dictionary)
#             # ponemos atributos
#             keys.update({'parent': parent, 'readonly': kwargs.get('readonly', True)})
#             # creamos la clase
#             cls = model.cc.Class(**keys)
#             dictionary.update({cls.name: cls})
#             # buscamos los tipos definidos en la clase
#             types = _class.find('types')
#             for type in types:
#                 keys = {}
#                 for x in type.keys():
#                     keys[x] = type.get(x).format(**dictionary)
#                 keys.update({'parent': cls})
#                 t = model.cc.Type(**keys)
#                 dictionary.update({t.name: t})
#             # buscamos los miembros
#             members = _class.find("members")
#             for member in members:
#                 keys = {}
#                 for x in member.keys():
#                     keys[x] = member.get(x).format(**dictionary)
#                     if x in ['type', 'static', 'const', 'volatile', 'virtual', 'ptr', 'ref', 'mutable']:
#                         keys[x] = eval(keys[x], dictionary)
#                 keys['type'] = model.cc.typeinst(**keys)
#                 keys.update({'parent': cls})
#                 model.cc.MemberData(**keys)
#             # constructores
#             ctors = _class.find('ctors')
#             for _ctor in ctors:
#                 keys = {'parent': cls, 'name': cls.name, 'autoargs': False}
#                 for x in _ctor.keys():
#                     keys[x] = _ctor.get(x).format(**dictionary)
#                     if x == 'inline':
#                         keys[x] = eval(keys[x], dictionary)
#                 ctor = model.cc.Constructor(**keys)
#                 # argumentos
#                 args = _ctor.find('args')
#                 for arg in args:
#                     keys = {'parent': ctor}
#                     for x in arg.keys():
#                         keys[x] = arg.get(x).format(**dictionary)
#                         if x == 'type':
#                             keys[x] = eval(keys[x], dictionary)
#                     keys['type'] = model.cc.typeinst(**keys)
#                     model.cc.Argument(**keys)
# 
#             # buscamos los metodos
#             methods = _class.find('methods')
#             for _method in methods:
#                 keys = {'parent': cls}
#                 for x in _method.keys():
#                     k = keys[x] = _method.get(x).format(**dictionary)
#                     if x == 'type':
#                         if k:
#                             keys[x] = eval(k, dictionary)
#                         else:
#                             keys[x] = dictionary[k]  # implicit type
#                     elif k in ['True', 'False']:
#                         keys[x] = eval(k, dictionary)
# 
#                 contents = _method.findall('content')
#                 if contents is None:
#                     keys['content'] = ''
#                 else:
#                     best_match = -1
#                     best_content = None
#                     for content in contents:
#                         #verificamos el filtro
#                         f = content.get('filter', None)
#                         ok = True
#                         match = 0
#                         if f is not None:
#                             f = f.strip()[1:-1]  # quitamos los { y }
#                             fk = dict([(x.strip(), (y.strip() == 'True'))
#                                 for x, y in [list(v.split(':')) for v in f.split(',')]])
#                                 # para pasar el filtro, deben coincidir los valores
#                             for k in key:
#                                 if k in fk:
#                                     if key[k] != fk[k]:
#                                         ok = False
#                                         break
#                                     match += 1
#                         if ok and match > best_match:
#                             best_content = content
#                             best_match = match
#                     if best_content is None:
#                         keys['content'] = ''
#                     else:
#                         keys['content'] = best_content.findtext('.').format(**dictionary)
#                        
#                 keys['type'] = model.cc.typeinst(**keys)
#                 keys.update({'parent': cls})
# 
#                 method = model.cc.MemberMethod(**keys)
#                 # argumentos
#                 args = _method.find('args')
#                 for arg in args:
#                     keys = {}
#                     for x in arg.keys():
#                         keys[x] = arg.get(x).format(**dictionary)
#                         if x in ['type', 'static', 'const', 'volatile', 'ptr', 'ref']:
#                             keys[x] = eval(keys[x], dictionary)
#                     keys['type'] = model.cc.typeinst(**keys)
#                     keys.update({'parent': method})
#                     model.cc.Argument(**keys)
#             #destructor
#             _dtor = _class.find('dtor')
#             if _dtor:
#                 keys = {}
#                 for x in _dtor.keys():
#                     keys[x] = _dtor.get(x).format(**dictionary)
#                     if x in ['virtual', 'inline']:
#                         keys[x] = eval(keys[x], dictionary)
#                 keys.update({'parent': cls, 'name': cls.name})
#                 model.cc.Destructor(**keys)
# 
#     except Exception, e:
#         print('Exception {msg} failed reading {p}'.format(
#             msg=e.message, p=path))
#         return
